/*
 * Configuration manipulation functions for BR-6104K - Part of AMiLDA Linux
 * Copyright (C)2005 by Sergio Aguayo
 * (2007) modified for BR-6204Wg by W.Piotrzkowski
 *
 * Covered by the Mozilla Public License v1.1
 * See the license file for details.
 */
 
#include <stdlib.h>
#include <stdio.h>
#include "mtd.h"
#include "config.h"


APP_DESC config_table[] = {
    /*   ID			NAME			TYPE			OFFSET	    					SIZE				prt_size, 		prt_row_size */
    {SSID,			"SSID",			STRING_T,		0x06, /*OFFSET(ssid),*/				SIZE(ssid),			SIZE(ssid)+1, 		0},
    {CHANNEL,			"CHANNEL",		BYTE_T,			0x27, /*OFFSET(channel)*/			SIZE(channel),			PRT_SIZE_OF_BYTE_T, 	0},
    {ELAN_MAC_ADDR,		"ELAN_MAC_ADDR",	BYTE_ARRAY_T,		0x28, /*OFFSET(elan_mac_addr),*/		SIZE(elan_mac_addr), 		PRT_SIZE_OF_MAC_ADDR_T,	0},
    {WLAN_MAC_ADDR,		"WLAN_MAC_ADDR",	BYTE_ARRAY_T,		0x2E, /*OFFSET(wlan_mac_addr),*/		SIZE(wlan_mac_addr), 		PRT_SIZE_OF_MAC_ADDR_T, 0},
    {WEP,			"WEP",			BYTE_T,			0x34, /*OFFSET(wep),*/				SIZE(wep), 			PRT_SIZE_OF_BYTE_T,	0},
    {WEP64_KEY1,		"WEP64_KEY1",		BYTE_ARRAY_T,		0x35, /*OFFSET(wep64_key1),*/			SIZE(wep64_key1),		(SIZE(wep64_key1))*2+1,	0},
    {WEP64_KEY2,		"WEP64_KEY2",		BYTE_ARRAY_T,		0x3A, /*OFFSET(wep64_key2),*/			SIZE(wep64_key2),		(SIZE(wep64_key2))*2+1,	0},
    {WEP64_KEY3,		"WEP64_KEY3",		BYTE_ARRAY_T,		0x3F, /*OFFSET(wep64_key3),*/			SIZE(wep64_key3),		(SIZE(wep64_key3))*2+1,	0},
    {WEP64_KEY4,		"WEP64_KEY4",		BYTE_ARRAY_T,		0x44, /*OFFSET(wep64_key4),*/			SIZE(wep64_key4),		(SIZE(wep64_key4))*2+1,	0},
    {WEP128_KEY1,		"WEP128_KEY1",		BYTE_ARRAY_T,		0x49, /*OFFSET(wep128_key1),*/			SIZE(wep128_key1),		(SIZE(wep128_key1))*2+1,0},
    {WEP128_KEY2,		"WEP128_KEY2",		BYTE_ARRAY_T,		0x56, /*OFFSET(wep128_key2),*/			SIZE(wep128_key2),		(SIZE(wep128_key2))*2+1,0},
    {WEP128_KEY3,		"WEP128_KEY3",		BYTE_ARRAY_T,		0x63, /*OFFSET(wep128_key3),*/			SIZE(wep128_key3),		(SIZE(wep128_key3))*2+1,0},
    {WEP128_KEY4,		"WEP128_KEY4",		BYTE_ARRAY_T,		0x70, /*OFFSET(wep128_key4),*/			SIZE(wep128_key4),		(SIZE(wep128_key4))*2+1,0},
    {WEP_DEFAULT_KEY,		"WEP_DEFAULT_KEY",	BYTE_T,			0x7D, /*OFFSET(wep_default_key),*/		SIZE(wep_default_key),		PRT_SIZE_OF_BYTE_T,	0},
    {WEP_KEY_TYPE,		"WEP_KEY_TYPE",		BYTE_T,			0x7E, /*OFFSET(wep_key_type),*/			SIZE(wep_key_type),		PRT_SIZE_OF_BYTE_T,	0},
    {FRAG_THRESHOLD,		"FRAG_THRESHOLD",	WORD_T,			0x7F, /*OFFSET(frag_threshold),*/		SIZE(frag_threshold),		PRT_SIZE_OF_WORD_T,	0},
    {RTS_THRESHOLD,		"RTS_THRESHOLD",	WORD_T,			0x81, /*OFFSET(rts_threshold),*/		SIZE(rts_threshold),		PRT_SIZE_OF_WORD_T,	0},
    {SUPPORTED_RATES,		"SUPPORTED_RATES",	BYTE_T,			0x84, /*OFFSET(supported_rates),*/		SIZE(supported_rates),		PRT_SIZE_OF_BYTE_T,	0},
    {BASIC_RATES,		"BASIC_RATES",		BYTE_T,			0x86, /*OFFSET(basic_rates),*/			SIZE(basic_rates),		PRT_SIZE_OF_BYTE_T,	0},
    {BEACON_INTERVAL,		"BEACON_INTERVAL",	BYTE_T,			0x88, /*OFFSET(beacon_interval),*/		SIZE(beacon_interval),		PRT_SIZE_OF_BYTE_T,	0},
    {PREAMBLE_TYPE,		"PREAMBLE_TYPE",	BYTE_T,			0x89, /*OFFSET(preamble_type),*/		SIZE(preamble_type),		PRT_SIZE_OF_BYTE_T,	0},
    {AUTH_TYPE,			"AUTH_TYPE",		BYTE_T,			0x8A, /*OFFSET(auth_type),*/ 			SIZE(auth_type),		PRT_SIZE_OF_BYTE_T,	0},

    {WLAN_MACAC_ENABLED,	"WLAN_MACAC_ENABLED",	BYTE_T,			0x8C, /*OFFSET(wlan_macac_enabled),*/		SIZE(wlan_macac_enabled),	PRT_SIZE_OF_BYTE_T,	0},
    {WLAN_MACAC_NUM,		"WLAN_MACAC_NUM",	BYTE_T,			0x8D, /*OFFSET(wlan_macac_num),*/		SIZE(wlan_macac_num),		PRT_SIZE_OF_BYTE_T,	0},

    {HIDDEN_SSID,		"HIDDEN_SSID",		BYTE_T,			0x259, /*OFFSET(hidden_ssid),*/			SIZE(hidden_ssid),		PRT_SIZE_OF_BYTE_T,	0},
    {WLAN_DISABLED,		"WLAN_DISABLED",	BYTE_T,			0x25A, /*OFFSET(wlan_disabled),*/		SIZE(wlan_disabled),		PRT_SIZE_OF_BYTE_T,	0},
    {INACTIVITY_TIME,		"INACTIVITY_TIME",	WORD_T,			0x25D, /*OFFSET(inactivity_time),*/		SIZE(inactivity_time),		PRT_SIZE_OF_WORD_T,	0},
    {RATE_ADAPTIVE_ENABLED,	"RATE_ADAPTIVE_ENABLED", BYTE_T,		0x25F, /*OFFSET(rate_adaptive_enabled),*/	SIZE(rate_adaptive_enabled),	PRT_SIZE_OF_BYTE_T,	0},
    {AP_MODE,			"AP_MODE",		BYTE_T,			0x260, /*OFFSET(ap_mode),*/			SIZE(ap_mode),			PRT_SIZE_OF_BYTE_T,	0},
    {IAPP_DISABLED,		"IAPP_DISABLED",	BYTE_T,			0x261, /*OFFSET(iapp_disabled),*/		SIZE(iapp_disabled),		PRT_SIZE_OF_BYTE_T,	0},
    {PROTECTION_DISABLED,	"PROTECTION_DISABLED",	BYTE_T,			0x262, /*OFFSET(protection_disabled),*/		SIZE(protection_disabled),	PRT_SIZE_OF_BYTE_T,	0},
    {MACCLONE_ENABLED,		"MACCLONE_ENABLED",	BYTE_T,			0x263, /*OFFSET(macclone_enabled),*/		SIZE(macclone_enabled),		PRT_SIZE_OF_BYTE_T,	0},
    {BAND,			"BAND",			BYTE_T,			0x264, /*OFFSET(band),*/			SIZE(band),			PRT_SIZE_OF_BYTE_T,	0},
    {FIX_RATE,			"FIX_RATE",		WORD_T,			0x265, /*OFFSET(fix_rate),*/			SIZE(fix_rate),			PRT_SIZE_OF_WORD_T,	0},
    {IP_ADDR,			"IP_ADDR",		IP_ADDR_T,		0x267, /*OFFSET(ip_addr),*/			SIZE(ip_addr),			PRT_SIZE_OF_IP_ADDR_T,	0},
    {SUBNET_MASK,		"SUBNET_MASK",		IP_ADDR_T,		0x26B, /*OFFSET(subnet_mask),*/			SIZE(subnet_mask),		PRT_SIZE_OF_IP_ADDR_T,	0},
    {DEFAULT_GATEWAY,		"DEFAULT_GATEWAY",	IP_ADDR_T,		0x26F, /*OFFSET(default_gateway),*/		SIZE(default_gateway),		PRT_SIZE_OF_IP_ADDR_T,	0},
    {DHCP,			"DHCP",			BYTE_T,			0x273, /*OFFSET(dhcp),*/			SIZE(dhcp),			PRT_SIZE_OF_BYTE_T,	0}, /* not sure */
    {STP_ENABLED,		"STP_ENABLED",		BYTE_T,			0x274, /*OFFSET(stp_enabled),*/			SIZE(stp_enabled),		PRT_SIZE_OF_BYTE_T,	0}, /* not sure */
    {SUPER_NAME,		"SUPER_NAME",		STRING_T,		0x275, /*OFFSET(super_name),*/			SIZE(super_name),		SIZE(super_name)+1,	0},
    {SUPER_PASSWORD,		"SUPER_PASSWORD",	STRING_T,		0x294, /*OFFSET(super_password),*/		SIZE(super_password),		SIZE(super_password)+1,	0},
    {USER_NAME,			"USER_NAME",		STRING_T,		0x2B3, /*OFFSET(user_name),*/			SIZE(user_name),		SIZE(user_name)+1,	0},
    {USER_PASSWORD,		"USER_PASSWORD",	STRING_T,		0x2D2, /*OFFSET(user_password),*/		SIZE(user_password),		SIZE(user_password)+1,	0},
    {DHCPGATEWAYIP_ADDR,	"DHCPGATEWAYIP_ADDR",	IP_ADDR_T,		0x2F1, /*OFFSET(dhcpgatewayip_addr),*/		SIZE(dhcpgatewayip_addr),	PRT_SIZE_OF_IP_ADDR_T,	0},
    {DHCPNAMESERVER_ADDR,	"DHCPNAMESERVER_ADDR",	IP_ADDR_T,		0x2F5, /*OFFSET(dhcpnameserver_addr),*/		SIZE(dhcpnameserver_addr),	PRT_SIZE_OF_IP_ADDR_T,	0},
    {DOMAIN_NAME,		"DOMAIN_NAME",		STRING_T,		0x2F9, /*OFFSET(domain_name),*/			SIZE(domain_name),		SIZE(domain_name)+1,	0},
    {DHCP_CLIENT_START,		"DHCP_CLIENT_START",	IP_ADDR_T,		0x318, /*OFFSET(dhcp_client_start),*/		SIZE(dhcp_client_start),	PRT_SIZE_OF_IP_ADDR_T,	0},
    {DHCP_CLIENT_END,		"DHCP_CLIENT_END",	IP_ADDR_T,		0x31C, /*OFFSET(dhcp_client_end),*/		SIZE(dhcp_client_end),		PRT_SIZE_OF_IP_ADDR_T,	0},
    {LAN_LEASE_TIME,		"LAN_LEASE_TIME",	LONG_T,			0x320, /*OFFSET(lan_lease_time),*/		SIZE(lan_lease_time),		PRT_SIZE_OF_LONG_T,	0},
    {DTIM_PERIOD,		"DTIM_PERIOD",		BYTE_T,			0x324, /*OFFSET(dtim_period),*/			SIZE(dtim_period),		PRT_SIZE_OF_BYTE_T,	0},
    {NETWORK_TYPE,		"NETWORK_TYPE",		BYTE_T,			0x325, /*OFFSET(network_type),*/		SIZE(network_type),		PRT_SIZE_OF_BYTE_T,	0},
    {WL_LINKMAC1,		"WL_LINKMAC1",		BYTE_ARRAY_T,		0x326, /*OFFSET(wl_linkmac1),*/			SIZE(wl_linkmac1),		PRT_SIZE_OF_MAC_ADDR_T,	0},
    {WL_LINKMAC2,		"WL_LINKMAC2",		BYTE_ARRAY_T,		0x32C, /*OFFSET(wl_linkmac2),*/			SIZE(wl_linkmac2),		PRT_SIZE_OF_MAC_ADDR_T,	0},
    {WL_LINKMAC3,		"WL_LINKMAC3",		BYTE_ARRAY_T,		0x332, /*OFFSET(wl_linkmac3),*/			SIZE(wl_linkmac3),		PRT_SIZE_OF_MAC_ADDR_T,	0},
    {WL_LINKMAC4,		"WL_LINKMAC4",		BYTE_ARRAY_T,		0x338, /*OFFSET(wl_linkmac4),*/			SIZE(wl_linkmac4),		PRT_SIZE_OF_MAC_ADDR_T,	0},
    {WL_LINKMAC5,		"WL_LINKMAC5",		BYTE_ARRAY_T,		0x33E, /*OFFSET(wl_linkmac5),*/			SIZE(wl_linkmac5),		PRT_SIZE_OF_MAC_ADDR_T,	0},
    {WL_LINKMAC6,		"WL_LINKMAC6",		BYTE_ARRAY_T,		0x344, /*OFFSET(wl_linkmac6),*/			SIZE(wl_linkmac6),		PRT_SIZE_OF_MAC_ADDR_T,	0},
    {WDS_ENABLED,		"WDS_ENABLED",		BYTE_T,			0x34A, /*OFFSET(wds_enabled),*/			SIZE(wds_enabled),		PRT_SIZE_OF_BYTE_T,	0},
    {WDS_ENCRYPT,		"WDS_ENCRYPT",		BYTE_T,			0x34B, /*OFFSET(wds_encrypt),*/			SIZE(wds_encrypt),		PRT_SIZE_OF_BYTE_T,	0},
    {WDS_WEP_FORMAT,		"WDS_WEP_FORMAT",	BYTE_T,			0x34C, /*OFFSET(wds_wep_format),*/		SIZE(wds_wep_format),		PRT_SIZE_OF_BYTE_T,	0},
    {WDS_WEP_KEY,		"WDS_WEP_KEY",		STRING_T,		0x34D, /*OFFSET(wds_wep_key),*/			SIZE(wds_wep_key),		SIZE(wds_wep_key)+1,	0},
    {WDS_PSK_FORMAT,		"WDS_PSK_FORMAT",	BYTE_T,			0x368, /*OFFSET(wds_psk_format),*/		SIZE(wds_psk_format),		PRT_SIZE_OF_BYTE_T,	0},
    {WDS_PSK,			"WDS_PSK",		STRING_T,		0x369, /*OFFSET(wds_psk),*/			SIZE(wds_psk),			SIZE(wds_psk)+1,	0},

    {SECURITY_MODE,		"SECURITY_MODE",	BYTE_T,			0x3AA, /*OFFSET(security_mode),*/		SIZE(security_mode),		PRT_SIZE_OF_BYTE_T,	0},
    {CLIENT_IP_DISABLED,	"CLIENT_IP_DISABLED",	BYTE_T,			0x3AB, /*OFFSET(client_ip_disabled),*/		SIZE(client_ip_disabled),	PRT_SIZE_OF_BYTE_T,	0},
    {DOT1X_MODE,		"DOT1X_MODE",		BYTE_T,			0x3AC, /*OFFSET(dot1x_mode),*/			SIZE(dot1x_mode),		PRT_SIZE_OF_BYTE_T,	0},

    {WLAN_SET_TX,		"WLAN_SET_TX",		BYTE_T,			0x3AE, /*OFFSET(wlan_set_tx),*/			SIZE(wlan_set_tx),		PRT_SIZE_OF_BYTE_T,	0},
    {WLAN_TRAN_RATE,		"WLAN_TRAN_RATE",	STRING_T,		0x3AF, /*OFFSET(wlan_tran_rate),*/		SIZE(wlan_tran_rate),		SIZE(wlan_tran_rate),	0},
    {WLAN_RATE_MODE,		"WLAN_RATE_MODE",	BYTE_T,			0x3B4, /*OFFSET(wlan_rate_mode),*/		SIZE(wlan_rate_mode),		PRT_SIZE_OF_BYTE_T,	0},
    {WLAN_CTS,			"WLAN_CTS",		BYTE_T,			0x3B5, /*OFFSET(wlan_cts),*/			SIZE(wlan_cts),			PRT_SIZE_OF_BYTE_T,	0},
    {WLAN_BURST,		"WLAN_BURST",		BYTE_T,			0x3B6, /*OFFSET(wlan_burst),*/			SIZE(wlan_burst),		PRT_SIZE_OF_BYTE_T,	0},

    {PS_ENABLE,			"PS_ENABLE",		BYTE_T,			0x3F7, /*OFFSET(ps_enable),*/			SIZE(ps_enable),		PRT_SIZE_OF_BYTE_T,	0},
    {PS_IPPENABLE,		"PS_IPPENABLE",		BYTE_T,			0x3F8, /*OFFSET(ps_ippenable),*/		SIZE(ps_ippenable),		PRT_SIZE_OF_BYTE_T,	0},
    {PS_LPRENABLE,		"PS_LPRENABLE",		BYTE_T,			0x3F9, /*OFFSET(ps_lprenable),*/		SIZE(ps_lprenable),		PRT_SIZE_OF_BYTE_T,	0},
    {PS_NAME,			"PS_NAME",		STRING_T,		0x3FA, /*OFFSET(ps_name),*/			SIZE(ps_name),			SIZE(ps_name),		0},
    {PS_PORT1NAME,		"PS_PORT1NAME",		STRING_T,		0x42A, /*OFFSET(ps_port1name),*/		SIZE(ps_port1name),		SIZE(ps_port1name),	0},
    {PS_PORT2NAME,		"PS_PORT2NAME",		STRING_T,		0x45A, /*OFFSET(ps_port2name),*/		SIZE(ps_port2name),		SIZE(ps_port2name),	0},

    {LICENCE,			"LICENCE",		BYTE_T,			0x48B, /*OFFSET(licence),*/			SIZE(licence),			PRT_SIZE_OF_BYTE_T,	0}, /* ADDRESS OVERLAP?? */

    {WLAN_BLOCK_RELAY,		"WLAN_BLOCK_RELAY",	BYTE_T,			0x48B, /*OFFSET(wlan_block_relay),*/ /* NOT SURE */	SIZE(wlan_block_relay), PRT_SIZE_OF_BYTE_T,	0},
    {AUTO_MAC_CLONE,		"AUTO_MAC_CLONE",	BYTE_T,			0x48C, /*OFFSET(auto_mac_clone),*/		SIZE(auto_mac_clone),		PRT_SIZE_OF_BYTE_T,	0},
    {OP_MODE,			"OP_MODE",		BYTE_T,			0x48D, /*OFFSET(op_mode),*/			SIZE(op_mode),			PRT_SIZE_OF_BYTE_T,	0},
    {WISP_WAN_ID,		"WISP_WAN_ID",		BYTE_T,			0x48E, /*OFFSET(wisp_wan_id),*/			SIZE(wisp_wan_id),		PRT_SIZE_OF_BYTE_T,	0},
    {WLAN_ENCRYPT,		"WLAN_ENCRYPT",		BYTE_T,			0x48F, /*FOFFSET(wlan_encrypt),*/		SIZE(wlan_encrypt),		PRT_SIZE_OF_BYTE_T,	0},
    {WLAN_ENABLE_SUPP_NONWPA,	"WLAN_ENABLE_SUPP_NONWPA",	BYTE_T,		0x490, /*OFFSET(wlan_enable_supp_nonwpa),*/	SIZE(wlan_enable_supp_nonwpa),	PRT_SIZE_OF_BYTE_T,	0},
    {WLAN_SUPP_NONWPA,		"WLAN_SUPP_NONWPA",	BYTE_T,			0x491, /*OFFSET(wlan_supp_nonwpa),*/		SIZE(wlan_supp_nonwpa),		PRT_SIZE_OF_BYTE_T,	0},
    {WLAN_WPA_AUTH,		"WLAN_WPA_AUTH",	BYTE_T,			0x492, /*OFFSET(wlan_wpa_auth),*/		SIZE(wlan_wpa_auth),		PRT_SIZE_OF_BYTE_T,	0},
    {WLAN_WPA_CIPHER_SUITE,	"WLAN_WPA_CIPHER_SUITE",	BYTE_T,		0x493, /*OFFSET(wlan_wpa_cipher_suite),*/	SIZE(wlan_wpa_cipher_suite),	PRT_SIZE_OF_BYTE_T,	0},
    {WLAN_WPA_PSK,		"WLAN_WPA_PSK",		STRING_T,		0x494, /*OFFSET(wlan_wpa_psk),*/		SIZE(wlan_wpa_psk),		SIZE(wlan_wpa_psk),	0},

    {WLAN_WPA_GROUP_REKEY_TIME,	"WLAN_WPA_GROUP_REKEY_TIME",	LONG_T,		0x4D5, /*OFFSET(wlan_wpa_group_rekey_time),*/	SIZE(wlan_wpa_group_rekey_time), PRT_SIZE_OF_LONG_T,	0},
    {RS_IP,			"RS_IP",		IP_ADDR_T,		0x4D9, /*OFFSET(rs_ip),*/			SIZE(rs_ip),			PRT_SIZE_OF_IP_ADDR_T,	0}, /*prawdopodobnie*/
    {RS_PORT,			"RS_PORT",		WORD_T,			0x4DD, /*OFFSET(rs_port),*/			SIZE(rs_port),			PRT_SIZE_OF_WORD_T,	0},
    {RS_PASSWORD,		"RS_PASSWORD",		STRING_T,		0x4DF, /*OFFSET(rs_password),*/			SIZE(rs_password),		SIZE(rs_password),	0},

    {WLAN_ENABLE_1X,		"WLAN_ENABLE_1X",	BYTE_T,			0x500, /*OFFSET(wlan_enable_1x),*/		SIZE(wlan_enable_1x),		PRT_SIZE_OF_BYTE_T,	0},
    {WLAN_PSK_FORMAT,		"WLAN_PSK_FORMAT",	BYTE_T,			0x501, /*OFFSET(wlan_psk_format),*/		SIZE(wlan_psk_format),		PRT_SIZE_OF_BYTE_T,	0},

    {ACCOUNT_RS_IP,		"ACCOUNT_RS_IP",	IP_ADDR_T,		0x502, /*OFFSET(account_rs_ip),*/		SIZE(account_rs_ip),		PRT_SIZE_OF_IP_ADDR_T,	0}, /*prawdopodobne*/
    {ACCOUNT_RS_PORT,		"ACCOUNT_RS_PORT",	WORD_T,			0x506, /*OFFSET(account_rs_port),*/		SIZE(account_rs_port),		PRT_SIZE_OF_WORD_T,	0},
    {ACCOUNT_RS_PASSWORD,	"ACCOUNT_RS_PASSWORD",	STRING_T,		0x508, /*OFFSET(account_rs_password),*/		SIZE(account_rs_password),	SIZE(account_rs_password),	0},
    {ACCOUNT_RS_UPDATE_ENABLED,	"ACCOUNT_RS_UPDATE_ENABLED",	BYTE_T,		0x528, /*OFFSET(account_rs_update_enabled),*/	SIZE(account_rs_update_enabled), PRT_SIZE_OF_BYTE_T,	0}, /*prawdopodobne*/
    {ACCOUNT_RS_UPDATE_DELAY,	"ACCOUNT_RS_UPDATE_DELAY",	WORD_T,		0x529, /*OFFSET(account_rs_update_delay),*/	SIZE(account_rs_update_delay),	PRT_SIZE_OF_WORD_T,	0},
    
    {ACCOUNT_RS_MAXRETRY,	"ACCOUNT_RS_MAXRETRY",	WORD_T,			0x52B, /*OFFSET(account_rs_maxretry),*/		SIZE(account_rs_maxretry),	PRT_SIZE_OF_WORD_T,	0}, /* CHECK for double address */
    {MAC_AUTH_ENABLED,		"MAC_AUTH_ENABLED",	BYTE_T,			0x52B, /*OFFSET(mac_auth_enabled),*/		SIZE(mac_auth_enabled),		PRT_SIZE_OF_BYTE_T,	0},
    
    {RS_MAXRETRY,		"RS_MAXRETRY",		BYTE_T,			0x52C, /*OFFSET(rs_maxretry),*/			SIZE(rs_maxretry),		PRT_SIZE_OF_BYTE_T,	0},
    {RS_INTERVAL_TIME,		"RS_INTERVAL_TIME",	WORD_T,			0x52D, /*OFFSET(rs_interval_time),*/		SIZE(rs_interval_time),		PRT_SIZE_OF_WORD_T,	0},

    {ACCOUNT_RS_ENABLED,	"ACCOUNT_RS_ENABLED",	BYTE_T,			0x530, /*OFFSET(account_rs_enabled),*/		SIZE(account_rs_enabled),	PRT_SIZE_OF_BYTE_T,	0}, /* wrong address */
    {ACCOUNT_RS_INTERVAL_TIME,	"ACCOUNT_RS_INTERVAL_TIME",	BYTE_T,		0x531, /*OFFSET(account_rs_interval_time),*/	SIZE(account_rs_interval_time),	PRT_SIZE_OF_BYTE_T,	0},

    {WPA2_PRE_AUTH,		"WPA2_PRE_AUTH",	BYTE_T,			0x532, /*OFFSET(wpa2_pre_auth),*/		SIZE(wpa2_pre_auth),		PRT_SIZE_OF_BYTE_T,	0},
    {WPA2_CIPHER_SUITE,		"WPA2_CIPHER_SUITE",	BYTE_T,			0x533, /*OFFSET(wpa2_cipher_suite),*/		SIZE(wpa2_cipher_suite),	PRT_SIZE_OF_BYTE_T,	0},

    {TIME_ZONE_SEL,		"TIME_ZONE_SEL",	BYTE_T,			0x535, /*OFFSET(time_zone_sel),*/		SIZE(time_zone_sel),		PRT_SIZE_OF_BYTE_T,	0},
    {START_MONTH,		"START_MONTH",		WORD_T,			0x536, /*OFFSET(start_month),*/			SIZE(start_month),		PRT_SIZE_OF_BYTE_T,	0},
    {START_DAY,			"START_DAY",		WORD_T,			0x538, /*OFFSET(start_day),*/			SIZE(start_day),		PRT_SIZE_OF_BYTE_T,	0},
    {END_MONTH,			"END_MONTH",		WORD_T,			0x53A, /*OFFSET(end_month),*/			SIZE(end_month),		PRT_SIZE_OF_BYTE_T,	0},
    {END_DAY,			"END_DAY",		WORD_T,			0x53C, /*OFFSET(end_day),*/			SIZE(end_day),			PRT_SIZE_OF_BYTE_T,	0},
    {TIMESERVER_ADDR,		"TIMESERVER_ADDR",	IP_ADDR_T,		0x53E, /*OFFSET(timeserver_addr),*/		SIZE(timeserver_addr),		PRT_SIZE_OF_IP_ADDR_T,	0},
    {DAYLIGHT_ENABLE,		"DAYLIGHT_ENABLE",	BYTE_T,			0x542, /*OFFSET(daylight_enable),*/		SIZE(daylight_enable),		PRT_SIZE_OF_BYTE_T,	0},
    {REMANHOST_ADDR,		"REMANHOST_ADDR",	IP_ADDR_T,		0x543, /*OFFSET(remanhost_addr),*/		SIZE(remanhost_addr),		PRT_SIZE_OF_IP_ADDR_T,	0},
    {REMAN_PORT,		"REMAN_PORT",		WORD_T,			0x547, /*OFFSET(reman_port),*/			SIZE(reman_port),		PRT_SIZE_OF_WORD_T,	0},
    {REMANG_ENABLE,		"REMANG_ENABLE",	BYTE_T,			0x549, /*OFFSET(remang_enable),*/		SIZE(remang_enable),		PRT_SIZE_OF_BYTE_T,	0}, /* OK */
    {WAN_MODE,			"WAN_MODE",		BYTE_T,			0x54A, /*OFFSET(wan_mode),*/			SIZE(wan_mode),			PRT_SIZE_OF_BYTE_T,	0}, /* OK */
    {WAN_DHCP,			"WAN_DHCP",		BYTE_T,			0x54B, /*OFFSET(wan_dhcp),*/			SIZE(wan_dhcp),			PRT_SIZE_OF_BYTE_T,	0},
    {WAN_IP_ADDR,		"WAN_IP_ADDR",		IP_ADDR_T,		0x54C, /*OFFSET(wan_ip_addr),*/			SIZE(wan_ip_addr),		PRT_SIZE_OF_IP_ADDR_T,	0},
    {WAN_SUBNET_MASK,		"WAN_SUBNET_MASK",	IP_ADDR_T,		0x550, /*OFFSET(wan_subnet_mask(,*/		SIZE(wan_subnet_mask),		PRT_SIZE_OF_IP_ADDR_T,	0},
    {WAN_DEFAULT_GATEWAY,	"WAN_DEFAULT_GATEWAY",	IP_ADDR_T,		0x554, /*OFFSET(wan_default_gateway),*/		SIZE(wan_default_gateway),	PRT_SIZE_OF_IP_ADDR_T,	0},
    {DNS_MODE,			"DNS_MODE",		BYTE_T,			0x558, /*OFFSET(dns_mode),*/			SIZE(dns_mode),			PRT_SIZE_OF_BYTE_T,	0},
    {DNS1,			"DNS1",			IP_ADDR_T,		0x55C, /*OFFSET(dns1),*/			SIZE(dns1),			PRT_SIZE_OF_IP_ADDR_T,	0},
    {DNS2,			"DNS2",			IP_ADDR_T,		0x560, /*OFFSET(dns2),*/			SIZE(dns2),			PRT_SIZE_OF_IP_ADDR_T,	0},
    {DNS3,			"DNS3",			IP_ADDR_T,		0x564, /*OFFSET(dns3),*/			SIZE(dns3),			PRT_SIZE_OF_IP_ADDR_T,	0},
    {WAN_MAC_ADDR,		"WAN_MAC_ADDR",		BYTE_ARRAY_T,		0x568, /*OFFSET(wan_mac_addr),*/		SIZE(wan_mac_addr),		PRT_SIZE_OF_MAC_ADDR_T,	0},
    {DYNIP_HOSTNAME,		"DYNIP_HOSTNAME",	STRING_T,		0x56E, /*OFFSET(dynip_hostname),*/		SIZE(dynip_hostname),		SIZE(dynip_hostname),	0},
    {PPP_USER_NAME,		"PPP_USER_NAME",	STRING_T,		0x58D, /*OFFSET(ppp_user_name),*/		SIZE(ppp_user_name),		SIZE(ppp_user_name),	0},
    {PPP_PASSWORD,		"PPP_PASSWORD",		STRING_T,		0x5CE, /*OFFSET(ppp_password),*/		SIZE(ppp_password),		SIZE(ppp_password),	0},
    {PPP_IDLE_TIME,		"PPP_IDLE_TIME",	WORD_T,			0x60F, /*OFFSET(ppp_idle_time),*/		SIZE(ppp_idle_time),		PRT_SIZE_OF_WORD_T,	0},
    {PPP_CONNECT_TYPE,		"PPP_CONNECT_TYPE",	BYTE_T,			0x611, /*OFFSET(ppp_connect_type),*/		SIZE(ppp_connect_type),		PRT_SIZE_OF_BYTE_T,	0},
    {PPP_SERVNAME,		"PPP_SERVNAME",		STRING_T,		0x612, /*OFFSET(ppp_servname),*/		SIZE(ppp_servname),		SIZE(ppp_servname),	0},
    {PPP_MTU,			"PPP_MTU",		WORD_T,			0x631, /*OFFSET(ppp_mtu),*/			SIZE(ppp_mtu),			PRT_SIZE_OF_WORD_T,	0},
    {PPTP_IPMODE,		"PPTP_IPMODE",		BYTE_T,			0x633, /*OFFSET(pptp_ipmode),*/			SIZE(pptp_ipmode),		PRT_SIZE_OF_BYTE_T,	0},
    {PPTP_IP_ADDR,		"PPTP_IP_ADDR",		IP_ADDR_T,		0x634, /*OFFSET(pptp_ip_addr),*/		SIZE(pptp_ip_addr),		PRT_SIZE_OF_IP_ADDR_T,	0},
    {PPTP_IP_MASK_ADDR,		"PPTP_IP_MASK_ADDR",	IP_ADDR_T,		0x638, /*OFFSET(pptp_ip_mask_addr),*/		SIZE(pptp_ip_mask_addr),	PRT_SIZE_OF_IP_ADDR_T,	0},
    {PPTP_DEF_GATEWAY,		"PPTP_DEF_GATEWAY",	IP_ADDR_T,		0x63C, /*OFFSET(pptp_def_gateway),*/		SIZE(pptp_def_gateway),		PRT_SIZE_OF_IP_ADDR_T,	0},
    {PPTP_GATEWAY,		"PPTP_GATEWAY",		IP_ADDR_T,		0x640, /*OFFSET(pptp_def_gateway),*/		SIZE(pptp_gateway),		PRT_SIZE_OF_IP_ADDR_T,	0},
    {PPTP_USER_NAME,		"PPTP_USER_NAME",	STRING_T,		0x644, /*OFFSET(pptp_user_name),*/		SIZE(pptp_user_name),		SIZE(pptp_user_name),	0},
    {PPTP_PASSWORD,		"PPTP_PASSWORD",	STRING_T,		0x685, /*OFFSET(pptp_password),*/		SIZE(pptp_password),		SIZE(pptp_password),	0},
    {PPTP_IDLE_TIME,		"PPTP_IDLE_TIME",	WORD_T,			0x6C6, /*OFFSET(pptp_idle_time),*/		SIZE(pptp_idle_time),		PRT_SIZE_OF_WORD_T,	0},
    {PPTP_CONNECT_TYPE,		"PPTP_CONNECT_TYPE",	BYTE_T,			0x6C8, /*OFFSET(pptp_connect_type),*/		SIZE(pptp_connect_type),	PRT_SIZE_OF_BYTE_T,	0},
    {PPTP_CONNT_ID,		"PPTP_CONNT_ID",	STRING_T,		0x6C9, /*OFFSET(pptp_connt_id),*/		SIZE(pptp_connt_id),		SIZE(pptp_connt_id),	0},
    {PPTP_BEZEQ_ENABLED,	"PPTP_BEZEQ_ENABLED",	BYTE_T,			0x6E8, /*OFFSET(pptp_bezeq_enabled),*/		SIZE(pptp_bezeq_enabled),	PRT_SIZE_OF_BYTE_T,	0},
    {PPTP_MTU,			"PPTP_MTU",		WORD_T,			0x6E9, /*OFFSET(pptp_mtu),*/			SIZE(pptp_mtu),			PRT_SIZE_OF_WORD_T,	0},
    {L2TP_IPMODE,		"L2TP_IPMODE",		BYTE_T,			0x6EB, /*OFFSET(l2tp_ipmode),*/			SIZE(l2tp_ipmode),		PRT_SIZE_OF_BYTE_T,	0},
    {L2TP_IP_ADDR,		"L2TP_IP_ADDR",		IP_ADDR_T,		0x6EC, /*OFFSET(l2tp_ip_addr),*/		SIZE(l2tp_ip_addr),		PRT_SIZE_OF_IP_ADDR_T,	0},
    {L2TP_MASK_ADDR,		"L2TP_MASK_ADDR",	IP_ADDR_T,		0x6F0, /*OFFSET(l2tp_mask_addr),*/		SIZE(l2tp_mask_addr),		PRT_SIZE_OF_IP_ADDR_T,	0},
    {L2TP_DEFGATEWAY,		"L2TP_DEFGATEWAY",	IP_ADDR_T,		0x6F4, /*OFFSET(l2tp_defgateway),*/		SIZE(l2tp_defgateway),		PRT_SIZE_OF_IP_ADDR_T,	0},
    {L2TP_GATEWAY,		"L2TP_GATEWAY",		STRING_T,		0x700, /*OFFSET(l2tp_gateway),*/		SIZE(l2tp_gateway),		PRT_SIZE_OF_IP_ADDR_T,	0},
    {L2TP_USER_NAME,		"L2TP_USER_NAME",	STRING_T,		0x717, /*OFFSET(l2tp_user_name),*/		SIZE(l2tp_user_name),		SIZE(l2tp_user_name),	0},
    {L2TP_PASSWORD,		"L2TP_PASSWORD",	STRING_T,		0x758, /*OFFSET(l2tp_password),*/		SIZE(l2tp_password),		SIZE(l2tp_password),	0},
    {L2TP_IDLE_TIME,		"L2TP_IDLE_TIME",	WORD_T,			0x799, /*OFFSET(l2tp_idle_time),*/		SIZE(l2tp_idle_time),		PRT_SIZE_OF_WORD_T,	0},
    {L2TP_CONNECT_TYPE,		"L2TP_CONNECT_TYPE",	BYTE_T,			0x79B, /*OFFSET(l2tp_connect_type),*/		SIZE(l2tp_connect_type),	PRT_SIZE_OF_BYTE_T,	0},
    {L2TP_MTU,			"L2TP_MTU",		WORD_T,			0x79C, /*OFFSET(l2tp_mtu),*/			SIZE(l2tp_mtu),			PRT_SIZE_OF_WORD_T,	0},
    {TELBP_IP_ADDR,		"TELBP_IP_ADDR",	IP_ADDR_T,		0x79E, /*OFFSET(telbp_ip_addr),*/		SIZE(telbp_ip_addr),		PRT_SIZE_OF_IP_ADDR_T,	0},
    {TELBP_USER_NAME,		"TELBP_USER_NAME",	STRING_T,		0x7A2, /*OFFSET(telbp_user_name),*/		SIZE(telbp_user_name),		SIZE(telbp_user_name),	0},
    {TELBP_PASSWORD,		"TELBP_PASSWORD",	STRING_T,		0x7E3, /*OFFSET(telbp_password),*/		SIZE(telbp_password),		SIZE(telbp_password),	0},
    {TELBP_ENABLED,		"TELBP_ENABLED",	BYTE_T,			0x824, /*OFFSET(telbp_enabled),*/		SIZE(telbp_enabled),		PRT_SIZE_OF_BYTE_T,	0},

    {DDNS_NAME,			"DDNS_NAME",		STRING_T,		0x825, /*OFFSET(ddns_name),*/				SIZE(ddns_name),		SIZE(ddns_name),	0},
    {DDNS_ACCOUNT,		"DDNS_ACCOUNT",		STRING_T,		0x844, /*OFFSET(ddns_account),*/		SIZE(ddns_account),		SIZE(ddns_account),	0},
    {DDNS_PASS,			"DDNS_PASS",		STRING_T,		0x863, /*OFFSET(ddns_pass),*/			SIZE(ddns_password),		SIZE(ddns_password),	0},
    {DDNS_ENABLED,		"DDNS_ENABLED",		BYTE_T,			0x882, /*OFFSET(ddns_enabled),*/		SIZE(ddns_enabled),		PRT_SIZE_OF_BYTE_T,	0},
    {DDNS_PVID_SEL,		"DDNS_PVID_SEL",	STRING_T,		0x883, /*OFFSET(ddns_pvid_sel),*/		SIZE(ddns_pvid_sel),		SIZE(ddns_pvid_sel),	0},

    {DHIS_HOSTID,		"DHIS_HOSTID",		BYTE_T,			0x890, /*OFFSET(dhis_hostid),*/			SIZE(dhis_hostid),		PRT_SIZE_OF_BYTE_T,	0},
    {DHIS_ISADDR,		"DHIS_ISADDR",		STRING_T,		0x891, /*OFFSET(dhis_isaddr),*/			SIZE(dhis_isaddr),		PRT_SIZE_OF_IP_ADDR_T,	0},
    {DHIS_PASSWORD,		"DHIS_PASSWORD",	STRING_T,		0x8B0, /*OFFSET(dhis_password),*/		SIZE(dhis_password),		SIZE(dhis_password),	0},
    {DHIS_AUTH_P1,		"DHIS_AUTH_P1",		STRING_T,		0x8CF, /*OFFSET(dhis_auth_p1),*/		SIZE(dhis_auth_p1),		SIZE(dhis_auth_p1),	0},
    {DHIS_AUTH_P2,		"DHIS_AUTH_P2",		STRING_T,		0x902, /*OFFSET(dhis_auth_p2),*/				SIZE(dhis_auth_p2),		SIZE(dhis_auth_p2),	0},
    {DHIS_AUTH_Q1,		"DHIS_AUTH_Q1",		STRING_T,		0x935, /*OFFSET(dhis_auth_q1),*/				SIZE(dhis_auth_q1),		SIZE(dhis_auth_q1),	0},
    {DHIS_AUTH_Q2,		"DHIS_AUTH_Q2",		STRING_T,		0x968, /*OFFSET(dhis_auth_q2),*/		SIZE(dhis_auth_q2),		SIZE(dhis_auth_q2),	0},
    {DHIS_SELECT,		"DHIS_SELECT",		BYTE_T,			0x99B, /*OFFSET(dhis_select),*/			SIZE(dhis_select),		PRT_SIZE_OF_BYTE_T,	0},
    {PORTFW_ENABLED,		"PORTFW_ENABLED",	BYTE_T,			0x99C, /*OFFSET(portfw_enabled),*/		SIZE(portfw_enabled),		PRT_SIZE_OF_BYTE_T,	0}, /* OK*/
    {PORTFW_TBL_NUM,		"PORTFW_TBL_NUM",	BYTE_T,			0x99D, /*OFFSET(portfw_tbl_num),*/		SIZE(portfw_tbl_num),		PRT_SIZE_OF_BYTE_T,	0}, /* OK */
    {PORTFW_TBL,		"PORTFW_TBL",		PORTFW_ARRAY_T,		0x99E, /*OFFSET(portfw_tbl),*/			SIZE(portfw_tbl),		16*(16+6+6+4+17+5+12),	(16+6+6+4+17+5+12)}, /*OK */
/* 0x0B58 */
    {IPFILTER_ENABLED,		"IPFILTER_ENABLED",	BYTE_T,			0xBA6, /*OFFSET(ipfilter_enabled),*/		SIZE(ipfilter_enabled),		PRT_SIZE_OF_BYTE_T,	0},
    {IPFILTER_TBL_NUM,		"IPFILTER_TBL_NUM",	BYTE_T,			0xBA8, /*OFFSET(ipfilter_tbl_num),*/		SIZE(ipfilter_tbl_num),		PRT_SIZE_OF_BYTE_T,	0}, /* wrong */
    {IPFILTER_TBL,		"IPFILTER_TBL",		IPFILTER_ARRAY_T,	0xBA9, 						SIZE(ipfilter_tbl),		16*(20),		20},
    
    {PORTFILTER_ENABLED,	"PORTFILTER_ENABLED",	BYTE_T,			0xBBE, /*OFFSET(portfilter_enabled),*/		SIZE(portfilter_enabled),	PRT_SIZE_OF_BYTE_T,	0},

    {MACFILTER_ENABLED,		"MACFILTER_ENABLED",	BYTE_T,			0xBD6, /*OFFSET(macfilter_enabled),*/		SIZE(macfilter_enabled),	PRT_SIZE_OF_BYTE_T,	0},
    {MACFILTER_TBL_NUM,		"MACFILTER_TBL_NUM",	BYTE_T,			0xBD7, /*OFFSET(macfilter_tbl_num),*/		SIZE(macfilter_tbl_num),	PRT_SIZE_OF_BYTE_T,	0},
    {MACFILTER_TBL,		"MACFILTER_TBL",	MACFILTER_ARRAY_T,	0xBD8, /*OFFSET(macfilter_tbl_num),*/		SIZE(macfilter_tbl),		36*16,			36 /*PRT_SIZE(macfilter_tbl_entry*/},

    {TRIGGERPORT_ENABLED,	"TRIGGERPORT_ENABLED",	BYTE_T,			0xDA4, /*OFFSET(triggerport_enabled),*/		SIZE(triggerport_enabled),	PRT_SIZE_OF_BYTE_T,	0},
    {TRIGGERPORT_TBL_NUM,	"TRIGGERPORT_TBL_NUM",	BYTE_T,			0xDA5, /*OFFSET(triggerport_tbl_num),*/		SIZE(triggerport_tbl_num),	PRT_SIZE_OF_BYTE_T,	0},
    {TRIGGERPORT_TBL,		"TRIGGERPORT_TBL",	TRIGGERPORT_ARRAY_T,	0xDA6, /*OFFSET(triggerport_tbl_num),*/		SIZE(triggerport_tbl),		16*100,			100}, /* 100 is arbitrary, TBD */

    {SROUT_ENABLED,		"SROUT_ENABLED",	BYTE_T,			0x112A, /*OFFSET(srout_enabled),*/		SIZE(srout_enabled),		PRT_SIZE_OF_BYTE_T,	0},
    {SROUT_TBL_NUM,		"SROUT_TBL_NUM",	BYTE_T,			0x112B, /*OFFSET(srout_tbl_num),*/		SIZE(srout_tbl_num),		PRT_SIZE_OF_BYTE_T,	0},
    {SROUT_TBL,			"SROUT_TBL",		SROUT_ARRAY_T,		0x112C, /*OFFSET(srout_tbl),*/			SIZE(srout_tbl),		16*100,			100}, /* 100 is arbitrary, TBD */

    {WAN1_QOS_TBL_NUM,		"WAN1_QOS_TBL_NUM",	BYTE_T,			0x1258, /*OFFSET(wan1_qos_tbl_num),*/		SIZE(wan1_qos_tbl_num),		PRT_SIZE_OF_BYTE_T,	0}, /* LONG SIZE?? */ /* 32 entries???? */
    {WAN1_QOS_ENABLED,		"WAN1_QOS_ENABLED",	BYTE_T,			0x125C, /*OFFSET(wan1_qos_enabled),*/		SIZE(wan1_qos_enabled),		PRT_SIZE_OF_BYTE_T,	0},
    {WAN1_QOS_TBL,		"WAN1_QOS_TBL",		WAN1_QOS_ARRAY_T,	0x1260, /*OFFSET(wan1_qos_tbl_num),*/		SIZE(wan1_qos_tbl),		32*200,			200}, /* 200 is arbitrary, TBD */
/* 1AF0 */
    {PORTFILTER_TBL_NUM,	"PORTFILTER_TBL_NUM",	BYTE_T,			0x2380, /* GUESSED!!!OFFSET(portfilter_tbl_num),*/SIZE(portfilter_tbl_num), 	PRT_SIZE_OF_BYTE_T,	0}, /*TO BE DEFINED!!*/
    {PORTFILTER_TBL,		"PORTFILTER_TBL",	PORTFILTER_ARRAY_T,	0x2381, /* SAME !! OFFSET(portfilter_tbl),*/	SIZE(portfilter_tbl),		16*(5+5+3+5+17+1),	(5+5+3+5+17+1)},
    {UPNP_ENABLE,		"UPNP_ENABLE",		BYTE_T,			0x2381, /*OFFSET(upnp_enable),*/		SIZE(upnp_enable),		PRT_SIZE_OF_BYTE_T,	0}, /* ?????????????? CHCECK DUPLICATE ADDDRESS */

    {VSER_ENABLED,		"VSER_ENABLED",		BYTE_T,			0x2381, /*OFFSET(vser_enabled),*/		SIZE(vser_enabled),		PRT_SIZE_OF_BYTE_T,	0},
    {VSER_TBL_NUM,		"VSER_TBL_NUM",		BYTE_T,			0x2382, /*OFFSET(vser_tbl_num),*/		SIZE(vser_tbl_num),		PRT_SIZE_OF_BYTE_T,	0},
    {VSER_TBL,			"VSER_TBL",		VSER_ARRAY_T,		0x2383, /*OFFSET(vser_tbl),*/			SIZE(vser_tbl),			16*100,			100}, /* 100 is arbitrary, TBD */

    {NAT_ENABLE,		"NAT_ENABLE",		BYTE_T,			0x258B, /*OFFSET(nat_enable),*/ 	SIZE(nat_enable),		PRT_SIZE_OF_BYTE_T,	0}, /* OK */
    {FIREWALL_ENABLE,		"FIREWALL_ENABLE",	BYTE_T,			0x258C, /*OFFSET(firewall_enable),*/	SIZE(firewall_enable),		PRT_SIZE_OF_BYTE_T,	0}, /* OK */
    {APP_LAYER_GATEWAY,		"APP_LAYER_GATEWAY",	ALG_BITMAP_T,		0x258D,	/*OFFSET(app_layer_gateway),*/	SIZE(app_layer_gateway), 	80,			0}, /* OK, BITMAP  original size is 4 bytes*/
    {MACDENY_ENABLED,		"MACDENY_ENABLED",	BYTE_T,			0x2591, /*OFFSET(macdeny_enabled),*/	SIZE(macdeny_enabled),		PRT_SIZE_OF_BYTE_T,	0}, /* OK */
    {IPDENY_ENABLED,		"IPDENY_ENABLED",	BYTE_T,			0x2592, /*OFFSET(ipdeny_enabled),*/	SIZE(ipdeny_enabled),		PRT_SIZE_OF_BYTE_T,	0}, /* OK */
    {ACPC_ENABLED,		"ACPC_ENABLED",		BYTE_T,			0x2593, /*OFFSET(acpc_enabled),*/	SIZE(acpc_enabled),		PRT_SIZE_OF_BYTE_T,	0}, /* OK */
    {ACPC_TBL_NUM,		"ACPC_TBL_NUM",		BYTE_T,			0x2594, /*OFFSET(acpc_tbl_num),*/	SIZE(acpc_tbl_num),		PRT_SIZE_OF_BYTE_T,	0}, /* OK Probably 20 entries*/
    {ACPC_TBL,			"ACPC_TBL",		ACPC_ARRAY_T,		0x2595, /*OFFSET(acpc_tbl_entry),*/	SIZE(acpc_tbl),			20*250,			250}, /* 50 is probably ok 150 is arbitrary, TBD, to be written 50?x20?*/

    {URLB_ENABLED,		"URLB_ENABLED",		BYTE_T,			0x297D, /*OFFSET(urlb_enabled),*/	SIZE(urlb_enabled),		PRT_SIZE_OF_BYTE_T,	0},
    {URLB_TBL_NUM,		"URLB_TBL_NUM",		BYTE_T,			0x297E, /*OFFSET(urlb_tbl_num),*/	SIZE(urlb_tbl_num),		PRT_SIZE_OF_BYTE_T,	0},
    {URLB_TBL,			"URLB_TBL",		URLB_ARRAY_T,		0x297F, /*OFFSET(urlb_tbl),*/		SIZE(urlb_tbl),			34*16,			34},

    {POD_ENABLED,		"POD_ENABLED",		BYTE_T,			0x2C27, /*OFFSET(pod_enabled),*/	SIZE(pod_enabled),		PRT_SIZE_OF_BYTE_T,	0}, /* OK */
    {PING_ENABLED,		"PING_ENABLED",		BYTE_T,			0x2C28, /*OFFSET(ping_enabled),*/	SIZE(ping_enabled),		PRT_SIZE_OF_BYTE_T,	0}, /* OK */
    {SCAN_ENABLED,		"SCAN_ENABLED",		BYTE_T,			0x2C29, /*OFFSET(scan_enabled),*/	SIZE(scan_enabled),		PRT_SIZE_OF_BYTE_T,	0}, /* OK */
    {SYN_ENABLED,		"SYN_ENABLED",		BYTE_T,			0x2C2A, /*OFFSET(syn_enabled),*/	SIZE(syn_enabled),		PRT_SIZE_OF_BYTE_T,	0}, /* OK */
    {POD_PACK,			"POD_PACK",		WORD_T,			0x2C2B, /*OFFSET(pod_pack),*/		SIZE(pod_pack),			PRT_SIZE_OF_WORD_T,	0}, /* OK */
    {POD_TIME,			"POD_TIME",		BYTE_T,			0x2C2D, /*OFFSET(pod_time),*/		SIZE(pod_time),			PRT_SIZE_OF_BYTE_T,	0}, /* OK */
    {POD_BUR,			"POD_BUR",		WORD_T,			0x2C2E, /*OFFSET(pod_bur),*/		SIZE(pod_bur),			PRT_SIZE_OF_WORD_T,	0}, /* OK */
    {SYN_PACK,			"SYN_PACK",		WORD_T,			0x2C30, /*OFFSET(syn_pack),*/		SIZE(syn_pack),			PRT_SIZE_OF_WORD_T,	0}, /* OK */
    {SYN_TIME,			"SYN_TIME",		BYTE_T,			0x2C32, /*OFFSET(syn_time),*/		SIZE(syn_time),			PRT_SIZE_OF_BYTE_T,	0}, /* OK */
    {SYN_BUR,			"SYN_BUR",		WORD_T,			0x2C33, /*OFFSET(syn_bur),*/		SIZE(syn_bur),			PRT_SIZE_OF_WORD_T,	0}, /* OK */
    {SCAN_NUM,			"SCAN_NUM",		SCAN_BITMAP_T,		0x2C35, /*OFFSET(scan_num),*/		SIZE(scan_num),			80,			0}, /* OK, original size is 4 bytes*/

    {DMZ_ENABLED,		"DMZ_ENABLED",		BYTE_T,			0x2C39, /*OFFSET(dmz_enabled),*/	SIZE(dmz_enabled),		PRT_SIZE_OF_BYTE_T,	0}, /* OK */
    {DMZ_TBL_NUM,		"DMZ_TBL_NUM",		BYTE_T,			0x2C3A, /*OFFSET(dmz_tbl_num),*/	SIZE(dmz_tbl_num),		PRT_SIZE_OF_BYTE_T,	0}, /* OK */
    {DMZ_TBL,			"DMZ_TBL",		DMZ_ARRAY_T,		0x2C3B, /*OFFSET(dmz_tbl_num),*/	SIZE(dmz_tbl),			16*25,			25}, /* OK */

    {PPP_TTL_ENABLED,		"PPP_TTL_ENABLED",	BYTE_T,			0x2D03, /*OFFSET(ppp_ttl_enabled),*/		SIZE(ppp_ttl_enabled),		PRT_SIZE_OF_BYTE_T,	0}, /* OK */
    {SDHCP_ENABLED,		"SDHCP_ENABLED",	BYTE_T,			0x2D04, /*OFFSET(sdhcp_enabled),*/		SIZE(sdhcp_enabled),		PRT_SIZE_OF_BYTE_T,	0}, /* OK */
    {SDHCP_TBL_NUM,		"SDHCP_TBL_NUM",	BYTE_T,			0x2D05, /*OFFSET(sdhcp_tbl_num),*/		SIZE(sdhcp_tbl_num),		PRT_SIZE_OF_BYTE_T,	0}, /* OK */
    {SDHCP_TBL,			"SDHCP_TBL",		DHCP_ARRAY_T,		0x2D06, /*OFFSET(sdhcp_tbl),*/			SIZE(sdhcp_tbl),		16*(18+16+1+9+1+3),	(18+16+1+9+1+3)}, /* OK */

    {MAX_DOWNLOAD_BANDWIDTH,	"MAX_DOWNLOAD_BANDWIDTH",	LONG_T,		0x2DA6, /*OFFSET(max_download_bandwidth),*/	SIZE(max_download_bandwidth),	PRT_SIZE_OF_LONG_T,	0},
    {MAX_UPLOAD_BANDWIDTH,	"MAX_UPLOAD_BANDWIDTH",	LONG_T,			0x2DAA, /*OFFSET(max_upload_bandwidth),*/ 	SIZE(max_upload_bandwidth),	PRT_SIZE_OF_LONG_T,	0},
    {REPEATER_ENABLED,		"REPEATER_ENABLED",	BYTE_T,			0x2DAE, /*OFFSET(repeater_enabled),*/		SIZE(repeater_enabled),		PRT_SIZE_OF_BYTE_T,	0},
    {REPEATER_SSID,		"REPEATER_SSID",	STRING_T,		0x2DAF, /*OFFSET(repeater_ssid),*/		SIZE(repeater_ssid),		SIZE(repeater_ssid),	0},
    {ROAMING_THREAD,		"ROAMING_THREAD",	BYTE_T,			0x2DD0, /*OFFSET(roaming_thread),*/	    	SIZE(roaming_thread),		PRT_SIZE_OF_BYTE_T,	0},

    {WEP152_KEY1,		"WEP152_KEY1",		BYTE_ARRAY_T,		OFFSET(wep152_key1),				SIZE(wep152_key1),		SIZE(wep152_key1),	0},
    {WEP152_KEY2,		"WEP152_KEY2",		BYTE_ARRAY_T,		OFFSET(wep152_key2),				SIZE(wep152_key2),		SIZE(wep152_key2),	0},
    {WEP152_KEY3,		"WEP152_KEY3",		BYTE_ARRAY_T,		OFFSET(wep152_key3),				SIZE(wep152_key3),		SIZE(wep152_key3),	0},
    {WEP152_KEY4,		"WEP152_KEY4",		BYTE_ARRAY_T,		OFFSET(wep152_key4),				SIZE(wep152_key4),		SIZE(wep152_key4),	0},
    {0}
};

CONFIG_STRUCT cfg;
HW_CONFIG_STRUCT hw;

/*
 * Function: sPrintIP_Addr
 *
 * Purpose:
 * reads 4 bytes from 1-st parameter and writes it as IP dotted notation to 2-nd param
 * both parameters must be pointers-to-pointers, 
 * at exit both pointers will be updated-> source +4, dest + len(printed addr).
 * memory must be allocated in calling procedure.
 */

int sPrintIP_Addr(unsigned char **source , unsigned char **dest) {
    char *row;
    row = malloc(PRT_SIZE_OF_IP_ADDR_T);
    sprintf(row, "%u.%u.%u.%u",
	(unsigned int)*(*source)++,
	(unsigned int)*(*source)++,
	(unsigned int)*(*source)++,
	(unsigned int)*(*source)++);
    strcpy(*dest, row);
    *dest = *dest + strlen(row);
    free(row);
    return 1;
	}
/*
 * Function: sPrintMacAddr
 *
 * Purpose:
 * reads 6 bytes from 1-st parameter and writes it as hex : separated 2-nd param
 * both parameters must be pointers-to-pointers, 
 * at exit both pointers will be updated-> source +6, dest + len(printed addr).
 * memory must be allocated in calling procedure.
 */

int sPrintMacAddr(unsigned char **source , unsigned char **dest) {
    char *row;
    row = malloc(PRT_SIZE_OF_MAC_ADDR_T);
    sprintf(row, "%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
	(unsigned int)*(*source)++,
	(unsigned int)*(*source)++,
	(unsigned int)*(*source)++,
	(unsigned int)*(*source)++,
	(unsigned int)*(*source)++,
	(unsigned int)*(*source)++);
    strcpy(*dest, row);
    *dest = *dest + strlen(row);
    free(row);
    return 1;
	}
/*
 * Function: sPrintWord
 *
 * Purpose:
 * reads 2 bytes from 1-st parameter and writes it as word to 2-nd param
 * both parameters must be pointers-to-pointers, used for port numbers
 * at exit both pointers will be updated-> source +2, dest + len(printed addr).
 * memory must be allocated in calling procedure.
 */

int sPrintWord(unsigned char **source , unsigned char **dest) {
    char *row;
    row = malloc(PRT_SIZE_OF_WORD_T);
    sprintf(row, "%u",	*(unsigned short*)*source);
    *source = *source + 2;
    strcpy(*dest, row);
    *dest = *dest + strlen(row);
    free(row);
    return 1;
	}
/*
 * Function: sPrintInt
 *
 * Purpose:
 * reads 1 byte from 1-st parameter and writes it as byte to 2-nd param
 * both parameters must be pointers-to-pointers, used for port numbers
 * at exit both pointers will be updated-> source +1, dest + len(printed value).
 * memory must be allocated in calling procedure.
 */

int sPrintInt(unsigned char **source , unsigned char **dest) {
    char *row;
    row = malloc(PRT_SIZE_OF_BYTE_T);
    sprintf(row, "%u",	*(unsigned char*)*source);
    *source = *source + 1;
    strcpy(*dest, row);
    *dest = *dest + strlen(row);
    free(row);
    return 1;
	}
/*
 * Function: sPrintLong
 *
 * Purpose:
 * reads 4 bytes from 1-st parameter and writes it as word to 2-nd param
 * both parameters must be pointers-to-pointers, used for LONG numbers
 * at exit both pointers will be updated-> source +4, dest + len(printed addr).
 * memory must be allocated in calling procedure.
 */

int sPrintLong(unsigned char **source , unsigned char **dest) {
    char *row;
    row = malloc(PRT_SIZE_OF_WORD_T);
    sprintf(row, "%u",	*(unsigned long*)*source);
    *source = *source + 4;
    strcpy(*dest, row);
    *dest = *dest + strlen(row);
    free(row);
    return 1;
	}
/*
 * Function: sPrintText
 *
 * Purpose:
 * reads text from 1-st parameter and writes it to 2-nd param
 * both parameters must be pointers-to-pointers, used for constant textes (NOT for comment field 
 * at exit only dest pointer will be updated->  dest + len(printed text).
 * memory must be allocated in calling procedure.
 */

int sPrintText(unsigned char *source , unsigned char **dest) {
    strcpy(*dest, source);
    *dest = *dest + strlen(source);
    return 1;
	}


/*
 * Function: createchecksum
 *
 * Purpose:
 * calculates checksum for variables stored in flash
 * 
 */

static inline unsigned char create_checksum(unsigned char *data, int len)
{
	int i;
	unsigned char sum=0;

	for (i=0; i<len; i++)
		if(i > 5) {sum += data[i];}  /* skip signature and size fields */
	sum = ~sum + 1;
	return sum;
}

/*
 * Function: chcecksum_OK
 *
 * Purpose:
 * verifies checksum for variables stored in flash
 * 
 */

static inline int checksum_OK(unsigned char *data, int len)
{
	int i;
	unsigned char sum=0;

	for (i=0; i<len; i++)
		if (i > 5) {sum += data[i];}  /* skip signature and size fields */
	if (sum == 0)
		return 1;
	else
		return 0;
}

/*
 * Function: configInit
 *
 * Purpose:
 * Loads the configuration from the flash rom. It also does some basic
 * consistency checking, though it doesn't verify the validity of the
 * parameters stored there.
 */
int configInit(void) {
/* debug */    
/*    printf("Config size is: %x\n", sizeof(CONFIG_STRUCT));*/
/* end debug W.P. */    
    if (!mtdRead(&cfg, sizeof(CONFIG_STRUCT)/* 0x7FFA*/, CONFIG_OFFSET)) {
	printf("Error reading config\n");
	return 0; }
    
    if (memcmp(cfg.signature, CONFIG_SIGNATURE, SIZE(signature)) != 0) {
	printf("Error reading signature\n");
	return 0; }

    if (!checksum_OK(&cfg, sizeof(cfg))) {
	printf("Error in chcecksum, please run flash reset/default\n");
	return 1; }

    if ((cfg.length+6) != sizeof(CONFIG_STRUCT)) { /* added 6 cause in Sergio's header */
	printf("Config size mismatch\n");          /* signature and length are included */
	return 1; }

    if (!mtdRead(&hw, sizeof(HW_CONFIG_STRUCT), HW_CONFIG_OFFSET)) {
	printf("Error reading hw structure\n");
	return 0; }
		
    return 1;
}

/*
 * Function: configUpdate
 *
 * Purpose:
 * Writes the loaded configuration back to the flash rom. It initializes
 * the header of the configuration before writing it.
 */
int configUpdate(void) {

    memcpy(cfg.signature, CONFIG_SIGNATURE, SIZE(signature));
    cfg.length = sizeof(CONFIG_STRUCT) - 6; /*header and size */
    cfg.checksum = create_checksum(&cfg, sizeof(cfg)-1);
    
    if (!mtdWrite(&cfg, sizeof(CONFIG_STRUCT), CONFIG_OFFSET))
	return 0;

    return 1;
}

/*
 * Function: configGet
 *
 * Purpose:
 * Reads a value from the loaded configuration, matched by ID.
 * changed from Sergio's version to support table values */
 
int configGet(int id, void *ptr) {
    int i = 0;
    unsigned char *dataptr = (unsigned char*)&cfg;
/* WP */
    void *data2;
    void *data;
    int datalen2;
    int datalen;
    int i2;
    int i3;
    int id2;
    char *row;
    char *row2;
    int row_size;
    int table_size;
    unsigned int temp_pointer;
    unsigned short temp_int;
    unsigned short temp_int2;
    unsigned char *temp_char;
    unsigned char tmp_char;
/* end WP */    
    do {
	if (config_table[i].id == id) {
	    temp_pointer = config_table[i].offset;
	    
	    switch (config_table[i].type) {
		case BYTE_T:
		    (*(unsigned char*)ptr) = dataptr[temp_pointer];
		    break;
		case WORD_T:
		    (*(unsigned short*)ptr) = *(unsigned short*)(&dataptr[temp_pointer]);
		    break;
		case STRING_T:
		    strcpy(ptr, &dataptr[temp_pointer]);
		    break;
		case IP_ADDR_T: 
		    sprintf(ptr, "%u.%u.%u.%u",
			(unsigned int)dataptr[temp_pointer+0],
			(unsigned int)dataptr[temp_pointer+1],
			(unsigned int)dataptr[temp_pointer+2],
			(unsigned int)dataptr[temp_pointer+3]);
		    break;
		case BYTE_ARRAY_T:
		    memcpy(ptr, &dataptr[temp_pointer], config_table[i].size);
		    break;
/* begin WP */
		case LONG_T:
		    sprintf(ptr, "%lu", *(unsigned long *)&dataptr[temp_pointer]);
		    break;
		    
		case DHCP_ARRAY_T:
		    /* ID we already have - it's a parameter */
		    id2 = configGetIDByString("SDHCP_TBL_NUM");
		    datalen2 = configGet_PRT_SizeByID(id2);
		    row_size = configGet_PRT_RowSizeByID(id);
		    data2 = malloc(datalen2);
		    if (data2 == NULL) return 0;
    		    if (!configGet(id2, data2)) {
			free(data2);
			return 0;   	        }
		    table_size = (unsigned int)(*(unsigned char*)data2);
		    if ( table_size == 0) { /* return OK if table is empty */
			free(data2);
			return 1;	  }

		    temp_char = &(dataptr[temp_pointer]);
		    for (i2 = 1; i2 <= table_size; i2++) {
			    row = malloc(row_size);
			    sprintf(row, "%s%u=",
			        configGetNameByID(id),
				i2); 
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    sPrintMacAddr(&temp_char, (unsigned char**)&ptr);
/* ,  */		    sPrintText((unsigned char *)&",", (unsigned char **)&ptr);
			    sPrintIP_Addr(&temp_char, (unsigned char**)&ptr);
/* \n */		    sPrintText((unsigned char *)&"\n", (unsigned char **)&ptr);
			    free(row);
						    }
		    free(data2);
		    break;

		case PORTFW_ARRAY_T:	
/* verified, works ok 2007-03-06, final version*/
		    id2 = configGetIDByString("PORTFW_TBL_NUM");
		    datalen2 = configGet_PRT_SizeByID(id2);
		    row_size = configGet_PRT_RowSizeByID(id);
		    data2 = malloc(datalen2);
		    if (data2 == NULL) return 0;
    		    if (!configGet(id2, data2)) {
			free(data2);
			return 0;   	        }
		    table_size = (unsigned int)(*(unsigned char*)data2);
		    if ( table_size == 0) { /* return OK if table is empty */
			free(data2);
			return 1;	  }

		    temp_char = &(dataptr[temp_pointer]);
		    for (i2 = 1; i2 <= table_size; i2++) {
			    row = malloc(row_size);
			    bzero(row, row_size);
			    sprintf(row, "%s%u=", 
			        configGetNameByID(id),
				i2);
			    strcpy(ptr, row);
			    ptr += strlen(row);
/* IP */		    sPrintIP_Addr(&temp_char, (unsigned char**)&ptr);
/* ,  */		    sPrintText((unsigned char *)&",", (unsigned char **)&ptr);
/* port_start */	    sPrintWord(&temp_char, (unsigned char **)&ptr);
/* ,  */ 		    sPrintText((unsigned char *)&",", (unsigned char **)&ptr);
/* port_end */		    sPrintWord(&temp_char, (unsigned char **)&ptr);
/* , */			    sPrintText((unsigned char *)&",", (unsigned char **)&ptr);
/* protocols */		    sPrintInt(&temp_char, (unsigned char **)&ptr);
/* ," */		    sPrintText((unsigned char *)&",\"", (unsigned char **)&ptr);
			    /*advance temp_point "manually" cause it is used later, (not temp_char)*/			    
			    temp_pointer += 9; 
			    bzero(row, row_size);
			    strcpy(row, &dataptr[temp_pointer]);
			    strcpy(ptr, row);
/* comment field*/	    ptr += strlen(row);
			    /* add CR */
/* "CR */		    sPrintText((unsigned char *)&"\"\n", (unsigned char **)&ptr);
			    /* advance source pointer by size of comment field */
			    temp_pointer += sizeof(cfg.portfw_tbl[0].comment);
			    temp_char += sizeof(cfg.portfw_tbl[0].comment);
			    free(row);
						    }
		    free(data2);
		    break;

		case VSER_ARRAY_T:
/* works OK, final version, 2007-03-15 */
		    id2 = configGetIDByString("VSER_TBL_NUM");
		    datalen2 = configGet_PRT_SizeByID(id2);
		    row_size = configGet_PRT_RowSizeByID(id);
		    data2 = malloc(datalen2);
		    if (data2 == NULL) return 0;
    		    if (!configGet(id2, data2)) {
			free(data2);
			return 0;   	        }
		    table_size = (unsigned int)(*(unsigned char*)data2);
		    if ( table_size == 0) { /* return OK if table is empty */
			free(data2);
			return 1;	  }

		    temp_char = &(dataptr[temp_pointer]);
		    for (i2 = 1; i2 <= table_size; i2++) {
			    row = malloc(row_size);
			    bzero(row, row_size);
			    
/* print row header */	    sprintf(row, "%s%u=", configGetNameByID(id), i2);
			    strcpy(ptr, row);
			    ptr += strlen(row);
/* private_IP */	    sPrintIP_Addr(&temp_char, (unsigned char **)&ptr);
/* ,  */ 		    sPrintText((unsigned char *)&",", (unsigned char **)&ptr);
/* private port */	    sPrintWord(&temp_char, (unsigned char **)&ptr);
/* ,  */ 		    sPrintText((unsigned char *)&",", (unsigned char **)&ptr);

			    temp_int = *(unsigned short*)temp_char;
/* public port */	    bzero(row, row_size);
			    sprintf(row, "%u,\"", temp_int);
			    temp_char++;
			    temp_char++;
/* protocols */		    sPrintInt(&temp_char, (unsigned char **)&ptr);
/* ,  */ 		    sPrintText((unsigned char *)&",", (unsigned char **)&ptr);
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    /* add comment */
			    bzero(row, row_size);
			    strcpy(row, temp_char);
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    /* add CR */
			    bzero(row, row_size);
			    sprintf(row, "\"\n");
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    /* advance source pointer by size of comment field */
			    temp_pointer += sizeof(cfg.vser_tbl[0].comment);
			    temp_char += sizeof(cfg.vser_tbl[0].comment);
			    free(row);
						    }
		    free(data2);
		    break;

		case SROUT_ARRAY_T:
/* verified, final version, 2007-03-15 */
		    id2 = configGetIDByString("SROUT_TBL_NUM");
		    datalen2 = configGet_PRT_SizeByID(id2);
		    row_size = configGet_PRT_RowSizeByID(id);
		    data2 = malloc(datalen2);
		    if (data2 == NULL) return 0;
    		    if (!configGet(id2, data2)) {
			free(data2);
			return 0;   	        }
		    table_size = (unsigned int)(*(unsigned char*)data2);
		    if ( table_size == 0) { /* return OK if table is empty */
			free(data2);
			return 1;	  }

		    temp_char = &(dataptr[temp_pointer]);
		    for (i2 = 1; i2 <= table_size; i2++) {
			    row = malloc(row_size);
			    
			    bzero(row, row_size);
/* print row header */	    sprintf(row, "%s%u=", configGetNameByID(id), i2);
			    strcpy(ptr, row);
			    ptr += strlen(row);
/* print IP address */	    sPrintIP_Addr(&temp_char, (unsigned char**)&ptr);
			    sPrintText((unsigned char *)&",", (unsigned char**)&ptr);
/* print IP netmask */	    sPrintIP_Addr(&temp_char, (unsigned char**)&ptr);
			    sPrintText((unsigned char *)&",", (unsigned char**)&ptr);
/* print gateway IP */	    sPrintIP_Addr(&temp_char, (unsigned char**)&ptr);
			    bzero(row, row_size);
			    tmp_char = *temp_char;
/* print to temporary */    sprintf(row, ",%u", tmp_char);
/* buffer */		    *temp_char++;
			    sPrintText((unsigned char *)&",", (unsigned char**)&ptr);
			    sPrintInt(&temp_char, (unsigned char**)&ptr);
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    sPrintText((unsigned char *)&"\n", (unsigned char**)&ptr);
			    
			    free(row);
						    }
		    free(data2);
		    break;

		case DMZ_ARRAY_T:
/* OK, NOT final version, 2007-03-15 */
		    id2 = configGetIDByString("DMZ_TBL_NUM");
		    datalen2 = configGet_PRT_SizeByID(id2);
		    row_size = configGet_PRT_RowSizeByID(id);
		    data2 = malloc(datalen2);
		    if (data2 == NULL) return 0;
    		    if (!configGet(id2, data2)) {
			free(data2);
			return 0;   	        }
		    table_size = (unsigned int)(*(unsigned char*)data2);
		    if ( table_size == 0) { /* return OK if table is empty */
			free(data2);
			return 1;	  }

		    for (i2 = 1; i2 <= table_size; i2++) {
			    row = malloc(row_size);
			    /* print flag1 */
			    bzero(row, row_size);
			    sprintf(row, "%s%u=%u,",
			        configGetNameByID(id),
				i2,
				*(unsigned char *)&dataptr[temp_pointer++]);
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    /* print flag2 */
			    bzero(row, row_size);
			    sprintf(row, "%u,",
				*(unsigned char *)&dataptr[temp_pointer++]);
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    /* print IP address1 */
			    bzero(row, row_size);
			    sprintf(row, "%u.%u.%u.%u,",
				(unsigned int)dataptr[temp_pointer++],
				(unsigned int)dataptr[temp_pointer++],
				(unsigned int)dataptr[temp_pointer++],
				(unsigned int)dataptr[temp_pointer++]);
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    /* print IP address2 */
			    bzero(row, row_size);
			    sprintf(row, "%u.%u.%u.%u\n",
				(unsigned int)dataptr[temp_pointer++],
				(unsigned int)dataptr[temp_pointer++],
				(unsigned int)dataptr[temp_pointer++],
				(unsigned int)dataptr[temp_pointer++]);
			    strcpy(ptr, row);
			    ptr += strlen(row);

			    free(row);
						    }
		    free(data2);
		    break;

		case URLB_ARRAY_T:
/* OK, final version */
		    id2 = configGetIDByString("URLB_TBL_NUM");
		    datalen2 = configGet_PRT_SizeByID(id2);
		    row_size = configGet_PRT_RowSizeByID(id);
		    data2 = malloc(datalen2);
		    if (data2 == NULL) return 0;
    		    if (!configGet(id2, data2)) {
			free(data2);
			return 0;   	        }
		    table_size = (unsigned int)(*(unsigned char*)data2);
		    if ( table_size == 0) { /* return OK if table is empty */
			free(data2);
			return 1;	  }

		    for (i2 = 1; i2 <= table_size; i2++) {
			    row = malloc(row_size);
			    /* print URLB_TBLxxx= */
			    bzero(row, row_size);
			    sprintf(row, "%s%u=",
			        configGetNameByID(id),
				i2);
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    /* print keyword */
			    bzero(row, row_size);
			    strcpy(row, &dataptr[temp_pointer]);
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    /* advance source pointer by size of comment field */
			    temp_pointer += sizeof(cfg.urlb_tbl[0].keyword);
			    /* add CR */
			    bzero(row, row_size);
			    sprintf(row, "\n");
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    
			    free(row);
						    }
		    free(data2);
		    break;

		case MACFILTER_ARRAY_T:
		    /* ID we already have - it's a parameter */
		    id2 = configGetIDByString("MACFILTER_TBL_NUM");
		    datalen2 = configGet_PRT_SizeByID(id2);
		    row_size = configGet_PRT_RowSizeByID(id);
		    data2 = malloc(datalen2);
		    if (data2 == NULL) return 0;
    		    if (!configGet(id2, data2)) {
			free(data2);
			return 0;   	        }
		    table_size = (unsigned int)(*(unsigned char*)data2);
		    if ( table_size == 0) { /* return OK if table is empty */
			free(data2);
			return 1;	  }

		    for (i2 = 1; i2 <= table_size; i2++) {
			    row = malloc(row_size);
			    /* print MAC address */
			    bzero(row, row_size);
			    sprintf(row, "%s%u=%.2X:%.2X:%.2X:%.2X:%.2X:%.2X,\"",
			        configGetNameByID(id),
				i2,
				(unsigned int)dataptr[temp_pointer++],
				(unsigned int)dataptr[temp_pointer++],
				(unsigned int)dataptr[temp_pointer++],
				(unsigned int)dataptr[temp_pointer++],
				(unsigned int)dataptr[temp_pointer++],
				(unsigned int)dataptr[temp_pointer++]);
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    /* print comment */
			    bzero(row, row_size);
			    strcpy(row, &dataptr[temp_pointer]);
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    /* add CR */
			    bzero(row, row_size);
			    sprintf(row, "\"\n");
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    /* advance source pointer by size of comment field */
			    temp_pointer += sizeof(cfg.macfilter_tbl[0].comment);
			    
			    free(row);
						    }
		    free(data2);
		    break;

		case TRIGGERPORT_ARRAY_T:
/* to be checked/corrected 2007-03-15 */
		    id2 = configGetIDByString("TRIGGERPORT_TBL_NUM");
		    datalen2 = configGet_PRT_SizeByID(id2);
		    row_size = configGet_PRT_RowSizeByID(id);
		    data2 = malloc(datalen2);
		    if (data2 == NULL) return 0;
    		    if (!configGet(id2, data2)) {
			free(data2);
			return 0;   	        }
		    table_size = (unsigned int)(*(unsigned char*)data2);
		    if ( table_size == 0) { /* return OK if table is empty */
			free(data2);
			return 1;	  }

		    for (i2 = 1; i2 <= table_size; i2++) {
			    row = malloc(row_size);
			    /* print port start */
			    bzero(row, row_size);
			    sprintf(row, "%s%u=%u-",
			        configGetNameByID(id),
				i2,
				*(unsigned short *)&dataptr[temp_pointer]);
				temp_pointer++;
				temp_pointer++;
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    /* print port end */
			    bzero(row, row_size);
			    sprintf(row, "%u:", 
				*(unsigned short *)&dataptr[temp_pointer]);
				temp_pointer++;
				temp_pointer++;
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    /* protocols */
			    bzero(row, row_size);
			    switch ((unsigned char)dataptr[temp_pointer++]) {
				case 1:
				    sprintf(row, "TCP");
				    break;
				case 2: 
				    sprintf(row, "UDP");
				    break;
				case 3:
				    sprintf(row, "ALL");
				    break;
											}
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    
			    bzero(row, row_size);
			    sprintf(row, ":%u-", 
				*(unsigned short *)&dataptr[temp_pointer]);
				temp_pointer++;
				temp_pointer++;
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    bzero(row, row_size);
			    sprintf(row, "%u", 
				*(unsigned short *)&dataptr[temp_pointer]);
				temp_pointer++;
				temp_pointer++;
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    bzero(row, row_size);
			    sprintf(row, ",%u-", 
				*(unsigned short *)&dataptr[temp_pointer]);
				temp_pointer++;
				temp_pointer++;
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    bzero(row, row_size);
			    sprintf(row, "%u", 
				*(unsigned short *)&dataptr[temp_pointer]);
				temp_pointer++;
				temp_pointer++;
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    bzero(row, row_size);
			    sprintf(row, ",%u-", 
				*(unsigned short *)&dataptr[temp_pointer]);
				temp_pointer++;
				temp_pointer++;
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    bzero(row, row_size);
			    sprintf(row, "%u", 
				*(unsigned short *)&dataptr[temp_pointer]);
				temp_pointer++;
				temp_pointer++;
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    bzero(row, row_size);
			    sprintf(row, ",%u-", 
				*(unsigned short *)&dataptr[temp_pointer]);
				temp_pointer++;
				temp_pointer++;
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    bzero(row, row_size);
			    sprintf(row, "%u", 
				*(unsigned short *)&dataptr[temp_pointer]);
				temp_pointer++;
				temp_pointer++;
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    bzero(row, row_size);
			    sprintf(row, ",%u-", 
				*(unsigned short *)&dataptr[temp_pointer]);
				temp_pointer++;
				temp_pointer++;
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    bzero(row, row_size);
			    sprintf(row, "%u:", 
				*(unsigned short *)&dataptr[temp_pointer]);
				temp_pointer++;
				temp_pointer++;
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    /* public protocols - maybe problem - size*/
			    temp_pointer++;
			    temp_pointer++;
			    bzero(row, row_size);
			    switch ((unsigned char)dataptr[temp_pointer++]) {
				case 1:
				    sprintf(row, "TCP");
				case 2: 
				    sprintf(row, "UDP");
				case 3:
				    sprintf(row, "ALL");
											}
			    strcpy(ptr, row);
			    ptr += strlen(row);

			    sprintf(row, "\n");
			    strcpy(ptr, row);
			    ptr += strlen(row);

			    temp_pointer += sizeof(cfg.triggerport_tbl[0].comment);

			    free(row);
						    }
		    free(data2);
		    break;

		case PORTFILTER_ARRAY_T: /* PORTFILTER is PROBABLY WRONG!! -> errors with Edimax flash utility */
		    /* ID we already have - it's a parameter */
		    id2 = configGetIDByString("PORTFILTER_TBL_NUM");
		    datalen2 = configGet_PRT_SizeByID(id2);
		    row_size = configGet_PRT_RowSizeByID(id);
		    data2 = malloc(datalen2);
		    if (data2 == NULL) return 0;
    		    if (!configGet(id2, data2)) {
			free(data2);
			return 0;   	        }
		    table_size = (unsigned int)(*(unsigned char*)data2);
		    if ( table_size == 0) { /* return OK if table is empty */
			free(data2);
			return 1;	  }

		    for (i2 = 1; i2 <= table_size; i2++) {
			    row = malloc(row_size);
			    /* print port start */
			    bzero(row, row_size);
			    sprintf(row, "%s%u=%u,",
			        configGetNameByID(id),
				i2,
				*(unsigned short *)&dataptr[temp_pointer]);
			    temp_pointer++;
			    temp_pointer++;
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    /* print port end */
			    bzero(row, row_size);
			    sprintf(row, "%u,",
				*(unsigned short *)&dataptr[temp_pointer]);
			    temp_pointer++;
			    temp_pointer++;
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    /* print protocols (???) */
			    bzero(row, row_size);
			    sprintf(row, "%u,\"",
				*(unsigned char *)&dataptr[temp_pointer++]);
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    /* print comment */
			    bzero(row, row_size);
			    strcpy(row, &dataptr[temp_pointer]);
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    /* add CR */
			    bzero(row, row_size);
			    sprintf(row, "\"\n");
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    /* advance source pointer by size of comment field */
			    temp_pointer += sizeof(cfg.portfilter_tbl[0].comment);
			    
			    free(row);
						    }
		    free(data2);
		    break;

		case SCAN_BITMAP_T:
/* 64 - synouport, 32 - synfin, 16 - synrst, 8 - nullscan, 4 - axmastree, 2 - xmastree */
/* 1 - nmap */
		    if (dataptr[temp_pointer+3] & 0x01) {
			sprintf(ptr, "nmap");
			ptr += 4;			}

		    if (dataptr[temp_pointer+3] & 0x02) {
			if (dataptr[temp_pointer+3] & 0x01) {
			    sprintf(ptr, ",");
			    ptr++;			    }
			sprintf(ptr, "xmastree");
			ptr += 8;			}

		    if (dataptr[temp_pointer+3] & 0x04) {
			if (dataptr[temp_pointer+3] & 0x03) {
			    sprintf(ptr, ",");
			    ptr++;			    }
			sprintf(ptr, "axmastree");
			ptr += 9;			}

		    if (dataptr[temp_pointer+3] & 0x08) {
			if (dataptr[temp_pointer+3] & 0x07) {
			    sprintf(ptr, ",");
			    ptr++;			    }
			sprintf(ptr, "nullscan");
			ptr += 8;			}

		    if (dataptr[temp_pointer+3] & 0x10) {
			if (dataptr[temp_pointer+3] & 0x0F) {
			    sprintf(ptr, ",");
			    ptr++;			    }
			sprintf(ptr, "synrst");
			ptr += 6;			}

		    if (dataptr[temp_pointer+3] & 0x20) {
			if (dataptr[temp_pointer+3] & 0x1F) {
			    sprintf(ptr, ",");
			    ptr++;			    }
			sprintf(ptr, "synfin");
			ptr += 6;			}

		    if (dataptr[temp_pointer+3] & 0x40) {
			if (dataptr[temp_pointer+3] & 0x3F) {
			    sprintf(ptr, ",");
			    ptr++;			    }
			sprintf(ptr, "synouport");
			ptr += 9;			}
						       
		    break;

		case ALG_BITMAP_T:
/* OK, 2007-03-15 */		
/* byte +1
/* 128 - talk, 64 - quake3, 32 - mms, 16 - irc, 8 - h323, 4 - ftp,2 - egg, 1 - amanda */
/* byte + 0 */
/* 4 - msn, 2 - starcraft, 1 - tftp */
		    if (dataptr[temp_pointer+3] & 0x01) {
			sprintf(ptr, "amanda");
			ptr += 6;			}

		    if (dataptr[temp_pointer+3] & 0x02) {
			if (dataptr[temp_pointer+3] & 0x01) {
			    sprintf(ptr, ",");
			    ptr++;			    }
			sprintf(ptr, "egg");
			ptr += 3;			}

		    if (dataptr[temp_pointer+3] & 0x04) {
			if (dataptr[temp_pointer+3] & 0x03) {
			    sprintf(ptr, ",");
			    ptr++;			    }
			sprintf(ptr, "ftp");
			ptr += 3;			}

		    if (dataptr[temp_pointer+3] & 0x08) {
			if (dataptr[temp_pointer+3] & 0x07) {
			    sprintf(ptr, ",");
			    ptr++;			    }
			sprintf(ptr, "h323");
			ptr += 4;			}

		    if (dataptr[temp_pointer+3] & 0x10) {
			if (dataptr[temp_pointer+3] & 0x0F) {
			    sprintf(ptr, ",");
			    ptr++;			    }
			sprintf(ptr, "irc");
			ptr += 3;			}

		    if (dataptr[temp_pointer+3] & 0x20) {
			if (dataptr[temp_pointer+3] & 0x1F) {
			    sprintf(ptr, ",");
			    ptr++;			    }
			sprintf(ptr, "mms");
			ptr += 3;			}

		    if (dataptr[temp_pointer+3] & 0x40) {
			if (dataptr[temp_pointer+3] & 0x3F) {
			    sprintf(ptr, ",");
			    ptr++;			    }
			sprintf(ptr, "quake3");
			ptr += 6;			}

		    if (dataptr[temp_pointer+3] & 0x80) {
			if (dataptr[temp_pointer+3] & 0x7F) {
			    sprintf(ptr, ",");
			    ptr++;			    }
			sprintf(ptr, "talk");
			ptr += 4;			}

		    if (dataptr[temp_pointer+2] & 0x01) {
			if (dataptr[temp_pointer+3] & 0xff) {
			    sprintf(ptr, ",");
			    ptr++;			    }
			sprintf(ptr, "tftp");
			ptr += 4;			}

		    if (dataptr[temp_pointer+2] & 0x02) {
			if ((dataptr[temp_pointer+2] & 0x01) |
			    (dataptr[temp_pointer+3] & 0xff)) {
			    sprintf(ptr, ",");
			    ptr++;			      }
			sprintf(ptr, "starcraft");
			ptr += 9;			}

		    if (dataptr[temp_pointer+2] & 0x04) {
			if ((dataptr[temp_pointer+2] & 0x03) |
			    (dataptr[temp_pointer+3] & 0xff)) {
			    sprintf(ptr, ",");
			    ptr++;			      }
			sprintf(ptr, "msn");
			ptr += 3;			}
						       
		    break;

		case ACPC_ARRAY_T:
		    /* ID we already have - it's a parameter */
/* as of 2007-03-20 seems right, known table structure, left minimal difference in row numbering when displaying */
		    id2 = configGetIDByString("ACPC_TBL_NUM");
		    datalen2 = configGet_PRT_SizeByID(id2);
		    row_size = configGet_PRT_RowSizeByID(id);
		    data2 = malloc(datalen2);
		    if (data2 == NULL) return 0;
    		    if (!configGet(id2, data2)) {
			free(data2);
			return 0;   	        }
		    table_size = (unsigned int)(*(unsigned char*)data2);
		    if ( table_size == 0) { /* return OK if table is empty */
			free(data2);
			return 1;	  }
		    temp_char = &(dataptr[temp_pointer]);
		    for (i2 = 1; i2 <= table_size; i2++) {
		    /* skip 2 byte entry number and 2byte bitmap field*/
			    temp_char += 2;
			    temp_int = *(unsigned short*)temp_char;
		    /* save bitmap to temporary variable */			    
			    temp_char += 2;
			    row = malloc(row_size);
			    row2 = malloc(row_size);
			    bzero(row, row_size);
			    bzero(row2, row_size);

  			    sprintf(row, "|%u.%u.%u.%u:%u.%u.%u.%u|",
				*(temp_char++),
				*(temp_char++),
				*(temp_char++),
				*(temp_char++),
				*(temp_char++),
				*(temp_char++),
				*(temp_char++),
				*(temp_char++));
			    tmp_char = *(unsigned char*)temp_char;
			    temp_char++;
			    for (i3 = 0; i3 <= 15; i3++) {
/* print row name and number */
				    bzero(row2, row_size);
				    switch (i3) {
					case 0: if (temp_int & 0x0001) { 
						    sprintf(row2, "%s%u=\'", configGetNameByID(id), i2); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
						    strcpy(ptr, row);
						    ptr += strlen(row);
						    bzero(row2, row_size);
						    sprintf(row2, "80,3128,8000,8080,8081|TCP|\'\n");
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
								       }
						break;
					case 1: if (temp_int & 0x0002) { 
						    sprintf(row2, "%s%u_%u=\'", configGetNameByID(id), i2, i3 ); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
						    strcpy(ptr, row);
						    ptr += strlen(row);
    						    bzero(row2, row_size);
						    sprintf(row2, "25|TCP|\'\n");
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
									}
						break;
					case 2: if (temp_int & 0x0004) {  
						    sprintf(row2, "%s%u_%u=\'", configGetNameByID(id), i2, i3 ); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
						    strcpy(ptr, row);
						    ptr += strlen(row);
    						    bzero(row2, row_size);
						    sprintf(row2, "119|TCP|\'\n"); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2); }
						break;
					case 3: if (temp_int & 0x0008) { 
						    sprintf(row2, "%s%u_%u=\'", configGetNameByID(id), i2, i3 ); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
						    strcpy(ptr, row);
						    ptr += strlen(row);
    						    bzero(row2, row_size);
						    sprintf(row2, "110|TCP|\'\n");
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
									 }
						break;
					case 4: if (temp_int & 0x0010)  { 
						    sprintf(row2, "%s%u_%u=\'", configGetNameByID(id), i2, i3 ); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
						    strcpy(ptr, row);
						    ptr += strlen(row);
    						    bzero(row2, row_size);
						    sprintf(row2, "443|TCP|\'\n");
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
							 }
						break;
					case 5: if (temp_int & 0x0020)  { 
						    sprintf(row2, "%s%u_%u=\'", configGetNameByID(id), i2, i3 ); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
						    strcpy(ptr, row);
						    ptr += strlen(row);
    						    bzero(row2, row_size);
						    sprintf(row2, "21|TCP|\'\n");
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
							 }
						break;
					case 6: if (temp_int & 0x0040)  { 
						    sprintf(row2, "%s%u_%u=\'", configGetNameByID(id), i2, i3 ); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
						    strcpy(ptr, row);
						    ptr += strlen(row);
    						    bzero(row2, row_size);
						    sprintf(row2, "1863|TCP|\'\n");
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
							 }
						break;
					case 7: if (temp_int & 0x0080)  { 
						    sprintf(row2, "%s%u_%u=\'", configGetNameByID(id), i2, i3 ); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
						    strcpy(ptr, row);
						    ptr += strlen(row);
    						    bzero(row2, row_size);
						    sprintf(row2, "23|TCP|\'\n");
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
							 }
						break;
					case 8: if (temp_int & 0x0100)   { 
						    sprintf(row2, "%s%u_%u=\'", configGetNameByID(id), i2, i3 ); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
						    strcpy(ptr, row);
						    ptr += strlen(row);
    						    bzero(row2, row_size);
						    sprintf(row2, "5190|TCP|\'\n");
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
							 }
						break;
					case 9: if (temp_int & 0x0200)   { 
						    sprintf(row2, "%s%u_%u=\'", configGetNameByID(id), i2, i3 ); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
						    strcpy(ptr, row);
						    ptr += strlen(row);
    						    bzero(row2, row_size);
    						    sprintf(row2, "389,522,1503,1720,1731|TCP|\'\n");
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
									}
						break;
					case 10: if (temp_int & 0x0400)  { 
						    sprintf(row2, "%s%u_%u=\'", configGetNameByID(id), i2, i3 ); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
						    strcpy(ptr, row);
						    ptr += strlen(row);
    						    bzero(row2, row_size);
						    sprintf(row2, "53|UDP|\'\n"); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
							}
						break;
					case 11: if (temp_int & 0x0800)  { 
						    sprintf(row2, "%s%u_%u=\'", configGetNameByID(id), i2, i3 ); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
						    strcpy(ptr, row);
						    ptr += strlen(row);
    						    bzero(row2, row_size);
						    sprintf(row2, "161,162|UDP|\'\n"); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
							}
						break;
					case 12: if (temp_int & 0x1000)   { 
						    sprintf(row2, "%s%u_%u=\'", configGetNameByID(id), i2, i3 ); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
						    strcpy(ptr, row);
						    ptr += strlen(row);
    						    bzero(row2, row_size);
						    sprintf(row2, "1723|TCP|\'\n"); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
							}
						break;
					case 13: if (temp_int & 0x2000)   { 
						    sprintf(row2, "%s%u_%u=\'", configGetNameByID(id), i2, i3 ); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
						    strcpy(ptr, row);
						    ptr += strlen(row);
    						    bzero(row2, row_size);
						    sprintf(row2, "1701|UDP|\'\n");
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
							 }
						break;
					case 14: if (temp_int & 0x4000)   { 
						    sprintf(row2, "%s%u_%u=\'", configGetNameByID(id), i2, i3 ); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
						    strcpy(ptr, row);
						    ptr += strlen(row);
    						    bzero(row2, row_size);
						    sprintf(row2, "1:65535|TCP|\'\n"); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
							}
						break;
					case 15: if (temp_int & 0x8000)   { 
						    sprintf(row2, "%s%u_%u=\'", configGetNameByID(id), i2, i3 ); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
						    strcpy(ptr, row);
						    ptr += strlen(row);
    						    bzero(row2, row_size);
						    sprintf(row2, "1:65535|UDP|\'\n"); 
						    strcpy(ptr, row2);
						    ptr += strlen(row2);
									    }
						break;
						}	
				}
			    temp_char += 17; /* skip protocols & comment */
			    for (i3 = 16; i3 < 21; i3++) {
				temp_int = *(unsigned short*)temp_char;
				temp_char++;
				temp_char++;
				temp_int2 = *(unsigned short*)temp_char;
				temp_char++;
				temp_char++;
				if ((temp_int != 0) & (temp_int2 != 0)) {
				    bzero(row2, row_size);
				    sprintf(row2, "%s%u_%u=\'", configGetNameByID(id), i2, i3); 
				    strcpy(ptr, row2);
				    ptr += strlen(row2);
				    strcpy(ptr, row);
				    ptr += strlen(row);
				    bzero(row2, row_size);
/* port1 start */	    	    sprintf(row2, "%u:%u|", temp_int, temp_int2);
				    strcpy(ptr, row2);
				    ptr += strlen(row2);

				    bzero(row2, row_size);				 
				    switch (tmp_char) {
					case 1:
					    sprintf(row2, "TCP|\'\n");
					    break;
					case 2: 
					    sprintf(row2, "UDP|\'\n");
					    break;
					case 3:
					    sprintf(row2, "ALL|\'\n");
					    break;
							}
				    strcpy(ptr, row2);
				    ptr += strlen(row2);
				}
			     }
 			    free(row);		
			    free(row2);	
			}	
		    free(data2);
		    break;

		case WAN1_QOS_ARRAY_T:
/* not sure it's right but seems working -> unknown/unused field at end */
/* final version 2007-03-16 */
		    id2 = configGetIDByString("WAN1_QOS_TBL_NUM");
		    datalen2 = configGet_PRT_SizeByID(id2);
		    row_size = configGet_PRT_RowSizeByID(id);
		    data2 = malloc(datalen2);
		    if (data2 == NULL) return 0;
    		    if (!configGet(id2, data2)) {
			free(data2);
			return 0;   	        }
		    table_size = (unsigned int)(*(unsigned char*)data2);
		    if ( table_size == 0) { /* return OK if table is empty */
			free(data2);
			return 1;	  }

		    temp_char = &(dataptr[temp_pointer]);
		    for (i2 = 1; i2 <= table_size; i2++) {
			    row = malloc(row_size);
/* print row name and number */
			    bzero(row, row_size);
			    sprintf(row, "%s%u=",
			        configGetNameByID(id),
				i2 );
			    strcpy(ptr, row);
			    ptr += strlen(row);

			    temp_char += 17; /* skip comments filed */
			    
/* upload */		    sPrintLong(&temp_char, (unsigned char**)&ptr);
/* ,  */		    sPrintText((unsigned char *)&",", (unsigned char **)&ptr);
			    sPrintInt(&temp_char, (unsigned char**)&ptr);
/* ,  */		    sPrintText((unsigned char *)&",", (unsigned char **)&ptr);
/* download */		    sPrintLong(&temp_char, (unsigned char**)&ptr);
/* ,  */		    sPrintText((unsigned char *)&",", (unsigned char **)&ptr);
			    sPrintInt(&temp_char, (unsigned char**)&ptr);
/* ,  */		    sPrintText((unsigned char *)&",", (unsigned char **)&ptr);
			    sPrintInt(&temp_char, (unsigned char**)&ptr);
/* ,  */		    sPrintText((unsigned char *)&",", (unsigned char **)&ptr);
/* IP1 start */		    sPrintIP_Addr(&temp_char, (unsigned char**)&ptr);
/* -  */		    sPrintText((unsigned char *)&"-", (unsigned char **)&ptr);
/* IP1 end */		    sPrintIP_Addr(&temp_char, (unsigned char**)&ptr);
/* ,  */		    sPrintText((unsigned char *)&",", (unsigned char **)&ptr);
/* MAC addr 1 */	    sPrintMacAddr(&temp_char, (unsigned char**)&ptr);
/* -  */		    sPrintText((unsigned char *)&"-", (unsigned char **)&ptr);
/* MAC addr 2 */	    sPrintMacAddr(&temp_char, (unsigned char**)&ptr);
/* -  */		    sPrintText((unsigned char *)&"-", (unsigned char **)&ptr);
/* MAC addr 3 */	    sPrintMacAddr(&temp_char, (unsigned char**)&ptr);
/* -  */		    sPrintText((unsigned char *)&"-", (unsigned char **)&ptr);
/* MAC addr 4 */	    sPrintMacAddr(&temp_char, (unsigned char**)&ptr);
/* -  */		    sPrintText((unsigned char *)&"-", (unsigned char **)&ptr);
/* MAC addr 5 */	    sPrintMacAddr(&temp_char, (unsigned char**)&ptr);
/* -  */		    sPrintText((unsigned char *)&"-", (unsigned char **)&ptr);
/* MAC addr 6 */	    sPrintMacAddr(&temp_char, (unsigned char**)&ptr);
/* -  */		    sPrintText((unsigned char *)&"-", (unsigned char **)&ptr);
/* MAC addr 7 */	    sPrintMacAddr(&temp_char, (unsigned char**)&ptr);
/* -  */		    sPrintText((unsigned char *)&"-", (unsigned char **)&ptr);
/* MAC addr 8 */	    sPrintMacAddr(&temp_char, (unsigned char**)&ptr);
/* ,  */		    sPrintText((unsigned char *)&",", (unsigned char **)&ptr);
/* IP2 start */		    sPrintIP_Addr(&temp_char, (unsigned char**)&ptr);
/* -  */		    sPrintText((unsigned char *)&"-", (unsigned char **)&ptr);
/* IP2 end */		    sPrintIP_Addr(&temp_char, (unsigned char**)&ptr);
			    bzero(row, row_size);
/* traffic type */		    switch (*(unsigned short*)(temp_char)) {
				case 0:
				    sprintf(row, ",None,");
				    break;
				case 1: 
				    sprintf(row, ",SMTP,");
				    break;
				case 2:
				    sprintf(row, ",HTTP,");
				    break;
				case 4:
				    sprintf(row, ",POP3,");
				    break;
				case 8:
				    sprintf(row, ",FTP,");
				    break;
								}
			    strcpy(ptr, row);
			    ptr += strlen(row);
			    temp_char++;	
			    temp_char++;
/* protocols? */	    sPrintInt(&temp_char, (unsigned char**)&ptr);
/* ,  */		    sPrintText((unsigned char *)&",", (unsigned char **)&ptr);
/* port_s1 */		    sPrintWord(&temp_char, (unsigned char**)&ptr);
/* :  */		    sPrintText((unsigned char *)&":", (unsigned char **)&ptr);
/* port_e1 */		    sPrintWord(&temp_char, (unsigned char**)&ptr);
/* -  */		    sPrintText((unsigned char *)&"-", (unsigned char **)&ptr);
/* port_s2 */		    sPrintWord(&temp_char, (unsigned char**)&ptr);
/* :  */		    sPrintText((unsigned char *)&":", (unsigned char **)&ptr);
/* port_e2 */		    sPrintWord(&temp_char, (unsigned char**)&ptr);
/* -  */		    sPrintText((unsigned char *)&"-", (unsigned char **)&ptr);
/* port_s3 */		    sPrintWord(&temp_char, (unsigned char**)&ptr);
/* :  */		    sPrintText((unsigned char *)&":", (unsigned char **)&ptr);
/* port_e3 */		    sPrintWord(&temp_char, (unsigned char**)&ptr);
/* -  */		    sPrintText((unsigned char *)&"-", (unsigned char **)&ptr);
/* port_s4 */		    sPrintWord(&temp_char, (unsigned char**)&ptr);
/* :  */		    sPrintText((unsigned char *)&":", (unsigned char **)&ptr);
/* port_e4 */		    sPrintWord(&temp_char, (unsigned char**)&ptr);
/* -  */		    sPrintText((unsigned char *)&"-", (unsigned char **)&ptr);
/* port_s5 */		    sPrintWord(&temp_char, (unsigned char**)&ptr);
/* :  */		    sPrintText((unsigned char *)&":", (unsigned char **)&ptr);
/* port_e5 */		    sPrintWord(&temp_char, (unsigned char**)&ptr);
/* ,  */		    sPrintText((unsigned char *)&",", (unsigned char **)&ptr);
/* port_s1 */		    sPrintWord(&temp_char, (unsigned char**)&ptr);
/* :  */		    sPrintText((unsigned char *)&":", (unsigned char **)&ptr);
/* port_e1 */		    sPrintWord(&temp_char, (unsigned char**)&ptr);
/* -  */		    sPrintText((unsigned char *)&"-", (unsigned char **)&ptr);
/* port_s2 */		    sPrintWord(&temp_char, (unsigned char**)&ptr);
/* :  */		    sPrintText((unsigned char *)&":", (unsigned char **)&ptr);
/* port_e2 */		    sPrintWord(&temp_char, (unsigned char**)&ptr);
/* -  */		    sPrintText((unsigned char *)&"-", (unsigned char **)&ptr);
/* port_s3 */		    sPrintWord(&temp_char, (unsigned char**)&ptr);
/* :  */		    sPrintText((unsigned char *)&":", (unsigned char **)&ptr);
/* port_e3 */		    sPrintWord(&temp_char, (unsigned char**)&ptr);
/* -  */		    sPrintText((unsigned char *)&"-", (unsigned char **)&ptr);
/* port_s4 */		    sPrintWord(&temp_char, (unsigned char**)&ptr);
/* :  */		    sPrintText((unsigned char *)&":", (unsigned char **)&ptr);
/* port_e4 */		    sPrintWord(&temp_char, (unsigned char**)&ptr);
/* -  */		    sPrintText((unsigned char *)&"-", (unsigned char **)&ptr);
/* port_s5 */		    sPrintWord(&temp_char, (unsigned char**)&ptr);
/* :  */		    sPrintText((unsigned char *)&":", (unsigned char **)&ptr);
/* port_e5 */		    sPrintWord(&temp_char, (unsigned char**)&ptr);
/* \n  */		    sPrintText((unsigned char *)&"\n", (unsigned char **)&ptr);

			    temp_char += 2; /* skip something??????? */

			free(row);
							}
		    free(data2);
		    break;

		case IPFILTER_ARRAY_T:
/* routine seems ok, offsets are wrong 2007-03.15 */		
		    /* ID we already have - it's a parameter */
		    id2 = configGetIDByString("IPFILTER_TBL_NUM");
		    datalen2 = configGet_PRT_SizeByID(id2);
		    row_size = configGet_PRT_RowSizeByID(id);
		    data2 = malloc(datalen2);
		    if (data2 == NULL) return 0;
    		    if (!configGet(id2, data2)) {
			free(data2);
			return 0;   	        }
		    table_size = (unsigned int)(*(unsigned char*)data2);
		    if ( table_size == 0) { /* return OK if table is empty */
			free(data2);
			return 1;	  }
		    temp_char = &(dataptr[temp_pointer]);
		    for (i2 = 1; i2 <= table_size; i2++) {
			    row = malloc(row_size);
/* print row name and number */
			    bzero(row, row_size);
			    sprintf(row, "%s%u=",
			        configGetNameByID(id),
				i2 );
			    strcpy(ptr, row);
			    ptr += strlen(row);
/* IP addr */		    sPrintIP_Addr(&temp_char, (unsigned char**)&ptr);
/* ,  */		    sPrintText((unsigned char *)&",", (unsigned char **)&ptr);
/* protocols */		    sPrintInt(&temp_char, (unsigned char**)&ptr);
/* \n  */		    sPrintText((unsigned char *)&"\n", (unsigned char **)&ptr);
								    }
		    free(data2);
		    break;
/* end W.P. */		    
		default:
		    return 0;
	    }
	    return 1;
	}
	i++;
    } while(config_table[i].id > 0);
    
    return 0;
}

/*
 * Function: configSet
 *
 * Purpose:
 * Modifies a parameter in the loaded configuration. If you want
 * to store the value to flash rom, you must call configUpdate
 * after calling this function.
 */
int configSet(int id, void *ptr, int size) {
    int i = 0;
    unsigned int n1, n2, n3, n4;
    unsigned char *dataptr = (unsigned char*)&cfg;
    
    if (ptr == NULL) return 0;
    
    do {
	if (config_table[i].id == id) {
	    switch (config_table[i].type) {
		case BYTE_T:
		    dataptr[config_table[i].offset] = *(unsigned char*)ptr;
		    break;
		case WORD_T:
		    *(unsigned short*)(&dataptr[config_table[i].offset]) = *(unsigned short*)ptr;
		    break;
		case STRING_T:
		    strcpy(dataptr+config_table[i].offset, ptr);
		    break;
		case IP_ADDR_T:
		    sscanf(ptr, "%u.%u.%u.%u", &n1, &n2, &n3, &n4);
		    dataptr[config_table[i].offset+0] = (unsigned char)n1;
		    dataptr[config_table[i].offset+1] = (unsigned char)n2;
		    dataptr[config_table[i].offset+2] = (unsigned char)n3;
		    dataptr[config_table[i].offset+3] = (unsigned char)n4;
		    break;
		case BYTE_ARRAY_T:
		    /* Should it use the size parameter or config_table[i].size? */
		    memcpy(&dataptr[config_table[i].offset], ptr, config_table[i].size);
		    break;
/* WP */
		case LONG_T:
		    *(unsigned long*)(&dataptr[config_table[i].offset]) = *(unsigned long*)ptr;
		    break;
/* end WP */
		default:
		    return 0;
	    }
	    return 1;
	}
	i++;
    } while (config_table[i].id > 0);

    return 0;
}


/*
 * Funcion: configCreateDefault
 *
 * Purpose:
 * Initialize the configuration block of the flash device with default values.
 */
int configCreateDefault(void) {
    
    memset(&cfg, 0, sizeof(cfg));
    
    cfg.ip_addr[0] = 192;
    cfg.ip_addr[1] = 168;
    cfg.ip_addr[2] = 1;
    cfg.ip_addr[3] = 1;
    
    cfg.subnet_mask[0] = 255;
    cfg.subnet_mask[1] = 255;
    cfg.subnet_mask[2] = 255;
    cfg.subnet_mask[3] = 0;
/*    
    cfg.lanDHCPEnable = 1;
    
    cfg.lanDHCPStart[0] = 192;
    cfg.lanDHCPStart[1] = 168;
    cfg.lanDHCPStart[2] = 1;
    cfg.lanDHCPStart[3] = 100;
    
    cfg.lanDHCPEnd[0] = 192;
    cfg.lanDHCPEnd[1] = 168;
    cfg.lanDHCPEnd[2] = 1;
    cfg.lanDHCPEnd[3] = 200;
    
    cfg.wanIPAddr[0] = 172;
    cfg.wanIPAddr[1] = 1;
    cfg.wanIPAddr[2] = 1;
    cfg.wanIPAddr[3] = 1;

    cfg.wanNetmask[0] = 255;
    cfg.wanNetmask[1] = 255;
    cfg.wanNetmask[2] = 255;
    cfg.wanNetmask[3] = 0;
    
    cfg.wanMTU = 1500;
        
    cfg.dnsServer1[0] = 0;
    cfg.dnsServer1[1] = 0;
    cfg.dnsServer1[2] = 0;    
    cfg.dnsServer1[3] = 0;

    cfg.dnsServer2[0] = 0;
    cfg.dnsServer2[1] = 0;
    cfg.dnsServer2[2] = 0;    
    cfg.dnsServer2[3] = 0;

    cfg.dnsServer3[0] = 0;
    cfg.dnsServer3[1] = 0;
    cfg.dnsServer3[2] = 0;    
    cfg.dnsServer3[3] = 0;

    cfg.fwEnable = 1;
    
    cfg.fwSynEnable = 1;
    cfg.fwSynRate = 5;
    cfg.fwSynTime = 0;
    cfg.fwSynBurst = 3;
    
    cfg.fwPodEnable = 1;
    cfg.fwPodRate = 5;
    cfg.fwPodTime = 0;
    cfg.fwPodBurst = 3;
    
    cfg.fwPSEnable = 1;
    cfg.fwPSLevel = 5;
    
    cfg.fwNATEnabled = 1;
    
    strcpy(cfg.usrLogin, "admin");
    strcpy(cfg.usrPassword, "1234");
    
    strcpy(cfg.hostName, "amilda");
    strcpy(cfg.hostDomain, "amilda");
    
    cfg.qosEnable = 0;
    strcpy(cfg.qosUplink, "0");
    strcpy(cfg.qosDownlink, "0");
    
    return configUpdate();    
    temporary disabled */
}

/*
 * Function: configGetIDByString
 *
 * Purpose:
 * Returns the parameter ID by matching a given string.
 */
int configGetIDByString (char *string) {
    int i = 0;
    
    while (config_table[i].id != 0) {
	if (!strcmp(config_table[i].name, string)) break;
	i++;
    }
    
    return config_table[i].id;
}

/*
 * Function: configGetTypeByID
 *
 * Purpose:
 * Returns the type associated with the parameter specified.
 */
int configGetTypeByID (int id) {
    int i = 0;
    while (config_table[i].id != 0) {
	if (config_table[i].id == id) break;
	i++;
    }    
    return config_table[i].type;
}

/*
 * Function: configGetNameByID
 *
 * Purpose:
 * Returns a pointer to the string representation of the given id.
 */
char *configGetNameByID (int id) {
    int i = 0;
    while (config_table[i].id != 0) {
	if (config_table[i].id == id) break;
	i++;
    }
    return config_table[i].name;
}

/*
 * Function: configGetSizeByID
 *
 * Purpose:
 * Returns the size of the specified entry identified by its ID.
 */
int configGetSizeByID (int id) {
    int i = 0;
    while (config_table[i].id != 0) {
	if (config_table[i].id == id) break;
	i++;
    }
    return config_table[i].size;
}

/*
 * Function: configGet_PRT_SizeByID
 *
 * Purpose:
 * Returns a PRINTABLE maximum size for the given id.
 ' in case of tables there will be printable size of ENTIRE array */
int configGet_PRT_SizeByID (int id) {
    int i = 0;
    while (config_table[i].id != 0) {
	if (config_table[i].id == id) break;
	i++;
    }
    return config_table[i].prt_size;
}

/*
 * Function: configGet_PRT_RowSizeByID
 *
 * Purpose:
 * Returns a PRINTABLE size of table row for the given id.
 */
int configGet_PRT_RowSizeByID (int id) {
    int i = 0;
    while (config_table[i].id != 0) {
	if (config_table[i].id == id) break;
	i++;
    }
    return config_table[i].prt_row_size;
}

/* --------------------------------------------------- */

char *configGetMACAddr0(void) {    
    return hw.nic0Addr;
}

char *configGetMACAddr1(void) {
    return hw.nic1Addr;
}

char *configGetWlanAddr(void) {
    return hw.wlanAddr;
}

char configGetBoardVersion(void) {
    return hw.boardVer;
}

char configGetRegDomain(void) {
    return hw.RegDomain;
}

char configGetRFType(void) {
    return hw.RFtype;
}

char configGetAntDiversity(void) {
    return hw.AntDiversity;
}

char configGetTxAnt(void) {
    return hw.TxAnt;
}

char configGetCCAmode(void) {
    return hw.CCAmode;
}

char configGetWlanLedType(void) {
    return hw.WlanLedType;
}

char configGetInitGain(void) {
    return hw.InitGain;
}

char *configGettxPowerCCK(void) {
    return hw.txPowerCCK;
}

char *configGettxPowerOFDM(void) {
    return hw.txPowerOFDM;
}

static int stringtobyte (char *string, char *bytearray) {
    int numchars = strlen(string)/2;
    int i;
    unsigned char c1, c2;
    int v;
    
    for (i = 0; i < numchars; i++) {
	c1 = *string;
	c2 = *(string+1);
	if (c1 >= '0' && c1 <= '9') c1 -= '0';
	else if (c1 >= 'A' && c1 <= 'F') c1 -= 0x37;
	else if (c1 >= 'a' && c1 <= 'f') c1 -= 0x57;
	else { printf("Unknown char.\n"); return 0; }
	if (c2 >= '0' && c2 <= '9') c2 -= '0';
	else if (c2 >= 'A' && c2 <= 'F') c2 -= 0x37;
	else if (c2 >= 'a' && c2 <= 'f') c2 -= 0x57;
	else { printf("Unknown char.\n"); return 0; }
	v = (c1<<4)+c2;
	string+=2;
	bytearray[i] = v;
    }
    
    return 1;
}

int configSetValueByString (int id, char *value) {
    int type, datalen;
    unsigned int a;
    unsigned char b;
    unsigned short c;
    unsigned long d;
    void *data;
    
    type = configGetTypeByID(id);
    
    switch (type) {
	case BYTE_T:
	    sscanf(value, "%u", &a);
	    b = a;
	    data = &b;
	    datalen = 1;
	    break;
	case WORD_T:
	    sscanf(value, "%u", &a);
	    c = a;
	    data = &c;
	    datalen = 2;
	    break;
	case STRING_T:
	case IP_ADDR_T:
	    data = value;
	    datalen = strlen(value)+1;
	    break;
	case BYTE_ARRAY_T:
	    datalen = strlen(value)/2;
	    data = malloc(datalen);
	    stringtobyte(value, data);
	    break;
/* WP */	    
	case LONG_T:
	    sscanf(value, "%lu", &a);
	    d = a;
	    data = &d;
	    datalen = 4;
	    break;
/* end WP */	    
	default:
	    return 0;
    }
    
    configSet(id, data, datalen);
    
    if (type == BYTE_ARRAY_T) free(data);
    
    return 1;
}
