/*
 * Head for configuration function for the BR-6104K - Part of AMiLDA Linux
 * Original author: David Hsu, davidhsu@realtek.com.tw
 * Copyright (C)2005 by Sergio Aguayo
 * (2007) modified for BR-6204Wg by W.Piotrzkowski
 *
 * Covered by the Mozilla Public License v1.1
 * See the license file for details
 */

#ifndef _CONFIG_H_
#define _CONFIG_H_

/*
	Note: the mtdWrite function will only write in the range
	from CONFIG_OFFSET to (CONFIG_OFFSET+MAX_CONFIG_SIZE).
*/
#define HW_CONFIG_OFFSET	0x6000
#define CONFIG_OFFSET		0x8000
#define DEFAULT_CONFIG_OFFSET	0x6400 /* area in flash looking like default settings */
#define MAX_CONFIG_SIZE		0x8000 /* because SIGNATURE and length is included in structure */

#define CONFIG_SIGNATURE	"DS01"

#define __PACK__	__attribute__ ((packed))

/*--------------------------------------------------------------------------------*/
/* Constants used for reading values from flash            			  */
/* for BR-6204Wg "guessed" from original (Edimax supplied) flash utility. W.P. 	  */
/*--------------------------------------------------------------------------------*/
#define SSID			    1
#define CHANNEL			    2
#define WEP			    3
#define WEP64_KEY1		    4
#define WEP64_KEY2		    5
#define WEP64_KEY3		    6
#define WEP64_KEY4		    7
#define WEP128_KEY1		    8
#define WEP128_KEY2		    9
#define WEP128_KEY3		   10
#define WEP128_KEY4		   11
#define WEP_DEFAULT_KEY 	   12
#define WEP_KEY_TYPE		   13
#define FRAG_THRESHOLD  	   14
#define SUPPORTED_RATES 	   15
#define BEACON_INTERVAL 	   16
#define PREAMBLE_TYPE		   17
#define BASIC_RATES		   18
#define RTS_THRESHOLD		   19
#define AUTH_TYPE		   20
#define HIDDEN_SSID		   21
#define WLAN_DISABLED		   22
#define	ELAN_MAC_ADDR		   23
#define WLAN_MAC_ADDR		   24
#define	WLAN_ENCRYPT		   25
#define	WLAN_ENABLE_SUPP_NONWPA	   26
#define	WLAN_SUPP_NONWPA	   27
#define	WLAN_WPA_AUTH		   28
#define	WLAN_WPA_CIPHER_SUITE	   29
#define	WLAN_WPA_PSK		   30
#define WLAN_WPA_GROUP_REKEY_TIME  31
#define	RS_IP			   32
#define	RS_PORT			   33
#define	RS_PASSWORD		   34
#define	WLAN_ENABLE_1X		   35
#define	WLAN_PSK_FORMAT		   36
#define INACTIVITY_TIME		   37
#define RATE_ADAPTIVE_ENABLED	   38
#define	WLAN_MACAC_ENABLED	   39
#define	WLAN_MACAC_NUM		   40
#define WLAN_MACAC_ADDR		   41 /* 4 entries??*/

#define SUPER_NAME		   45
#define SUPER_PASSWORD		   46
#define	USER_NAME		   47
#define USER_PASSWORD		   48
#define DHCP_CLIENT_START	   49
#define	DHCP_CLIENT_END		   50
#define	WAN_MAC_ADDR		   51
#define	WAN_IP_ADDR		   52
#define	WAN_SUBNET_MASK		   53
#define	WAN_DEFAULT_GATEWAY	   54
#define	WAN_DHCP		   55
#define	DNS_MODE		   56
#define	DNS1			   57
#define	DNS2			   58
#define	DNS3			   59
#define	PPP_USER_NAME		   60
#define	PPP_PASSWORD		   61
#define	PPP_IDLE_TIME		   62
#define	PPP_CONNECT_TYPE	   63
#define	PORTFW_ENABLED		   64
#define	PORTFW_TBL_NUM		   65
#define	PORTFW_TBL		   66

#define OP_MODE			   68
#define WISP_WAN_ID		   69
#define	IPFILTER_ENABLED	   70
#define	IPFILTER_TBL_NUM	   71
#define	IPFILTER_TBL		   72
#define	MACFILTER_ENABLED	   76
#define	MACFILTER_TBL_NUM	   77
#define	MACFILTER_TBL		   78 /* 4 entries??*/

#define	PORTFILTER_ENABLED	   82
#define	PORTFILTER_TBL_NUM	   83
#define	PORTFILTER_TBL		   84 /* 4 entries??*/

#define	TRIGGERPORT_ENABLED	   88
#define	TRIGGERPORT_TBL_NUM	   89
#define	TRIGGERPORT_TBL		   90 /* 4 entries??*/

#define	DMZ_ENABLED		   94
#define	WAN1_QOS_TBL_NUM	   95
#define	WAN1_QOS_TBL		   96 /* 7 entries??, or 7 fields??*/

#define	WAN1_QOS_ENABLED	  103

#define	IP_ADDR			  105
#define	SUBNET_MASK		  106
#define	DEFAULT_GATEWAY		  107
#define	DHCP			  108
#define	STP_ENABLED		  109
#define DTIM_PERIOD		  110
#define NETWORK_TYPE		  111
#define	DHCPGATEWAYIP_ADDR	  112
#define	DHCPNAMESERVER_ADDR	  113
#define	DOMAIN_NAME		  114
#define	LAN_LEASE_TIME		  115

#define	HW_BOARD_ID		  121
#define	HW_NIC0_ADDR		  122
#define	HW_NIC1_ADDR		  123
#define	HW_WLAN_ADDR		  124
#define	HW_REG_DOMAIN		  125
#define	HW_RF_TYPE		  126
#define	HW_TX_POWER_CCK		  127
#define	HW_TX_POWER_OFDM	  128
#define	HW_ANT_DIVERSITY	  129
#define	HW_TX_ANT		  130
#define	HW_CCA_MODE		  131
#define	HW_WLAN_LED		  132
#define	HW_INIT_GAIN		  133
#define	TIME_ZONE_SEL		  134
#define	START_MONTH		  135
#define	START_DAY		  136
#define	END_MONTH		  137
#define	END_DAY			  138
#define	TIMESERVER_ADDR		  139
#define	DAYLIGHT_ENABLE		  140
#define	REMANHOST_ADDR		  141
#define	REMANG_ENABLE		  142
#define	DYNIP_HOSTNAME		  143
#define PPTP_IP_ADDR		  144
#define	PPTP_IP_MASK_ADDR	  145
#define	PPTP_DEF_GATEWAY	  146
#define	PPTP_GATEWAY		  147
#define	PPTP_USER_NAME		  148
#define	PPTP_PASSWORD		  149
#define PPTP_IDLE_TIME		  150
#define	PPP_SERVNAME		  151
#define	PPP_MTU			  152
#define	NAT_ENABLE		  153
#define	FIREWALL_ENABLE		  154
#define	UPNP_ENABLE		  155
#define	VSER_ENABLED		  156
#define	VSER_TBL_NUM		  157
#define	VSER_TBL		  158 /* place for 4 entries?? */

#define	WAN_MODE		  162
#define	DMZ_TBL_NUM		  163
#define	DMZ_TBL			  164 /* place for 4 entries?? */ 

#define	POD_ENABLED		  168
#define	PING_ENABLED		  169
#define	URLB_ENABLED		  170
#define	URLB_TBL_NUM		  171
#define	URLB_TBL		  172
#define	ACPC_ENABLED		  176
#define	ACPC_TBL_NUM		  177
#define	ACPC_TBL		  178 /*8 entries??*/

#define	TELBP_IP_ADDR		  182
#define	TELBP_USER_NAME		  183
#define	TELBP_PASSWORD		  184
#define	TELBP_ENABLED		  185
#define	PPTP_IPMODE		  186
#define	DDNS_NAME		  187
#define	DDNS_ACCOUNT		  188
#define	DDNS_PASS		  189
#define	DDNS_ENABLED		  190
#define	DDNS_PVID_SEL		  191
#define	REMAN_PORT		  192
#define	PPTP_CONNECT_TYPE	  193
#define	L2TP_IPMODE		  194
#define	L2TP_IP_ADDR		  195
#define	L2TP_MASK_ADDR		  196
#define	L2TP_DEFGATEWAY		  197
#define	L2TP_GATEWAY		  198
#define	L2TP_USER_NAME		  199
#define	L2TP_PASSWORD		  200
#define	L2TP_IDLE_TIME		  201
#define	L2TP_CONNECT_TYPE	  202
#define	PPTP_CONNT_ID		  203
#define	PPTP_BEZEQ_ENABLED	  204
#define APP_LAYER_GATEWAY	  205
#define	SCAN_ENABLED		  206
#define	SYN_ENABLED		  207
#define	DHIS_HOSTID		  208
#define	DHIS_ISADDR		  209
#define	DHIS_PASSWORD		  210
#define	DHIS_AUTH_P1		  211
#define	DHIS_AUTH_P2		  212
#define	DHIS_AUTH_Q1		  213
#define	DHIS_AUTH_Q2		  214
#define	DHIS_SELECT		  215
#define	MAC_AUTH_ENABLED	  216
#define	RS_MAXRETRY		  217
#define	RS_INTERVAL_TIME	  218
#define	ACCOUNT_RS_ENABLED	  219
#define	ACCOUNT_RS_IP		  220
#define	ACCOUNT_RS_PORT		  221
#define	ACCOUNT_RS_PASSWORD	  222
#define	ACCOUNT_RS_UPDATE_ENABLED 223
#define	ACCOUNT_RS_UPDATE_DELAY	  224
#define	ACCOUNT_RS_MAXRETRY	  225
#define	ACCOUNT_RS_INTERVAL_TIME  226
#define IAPP_DISABLED		  227
#define AP_MODE			  228
#define	WL_LINKMAC1		  229
#define	WL_LINKMAC2		  230
#define	WL_LINKMAC3		  231
#define	WL_LINKMAC4		  232
#define	WL_LINKMAC5		  233
#define	WL_LINKMAC6		  234
#define SECURITY_MODE		  235
#define CLIENT_IP_DISABLED	  236
#define	SROUT_ENABLED		  237
#define	SROUT_TBL_NUM		  238
#define	SROUT_TBL		  239 /* 4 entries??, or 4 fields?? */

#define	DOT1X_MODE		  243
#define POD_PACK		  244
#define	POD_TIME		  245
#define	POD_BUR			  246
#define	SYN_PACK		  247
#define	SYN_TIME		  248
#define	SYN_BUR			  249
#define	SCAN_NUM		  250
#define	PPTP_MTU		  251
#define	L2TP_MTU		  252
#define WLAN_SET_TX		  253
#define	WDS_ENABLED		  254
#define	WDS_ENCRYPT		  255
#define	WDS_WEP_FORMAT		  256
#define	WDS_WEP_KEY		  257
#define	WDS_PSK_FORMAT		  258
#define	WDS_PSK			  259
#define	WLAN_TRAN_RATE		  260
#define	WLAN_RATE_MODE		  261
#define	WLAN_CTS		  262
#define	WLAN_BURST		  263
#define	WEP152_KEY1		  264
#define	WEP152_KEY2		  265
#define	WEP152_KEY3		  266
#define	WEP152_KEY4		  267 

#define	PS_ENABLE		  269
#define	PS_IPPENABLE		  271
#define	PS_LPRENABLE		  272
#define PS_NAME			  270
#define	PS_PORT1NAME		  273
#define	PS_PORT2NAME		  274 
#define	LICENCE			  275
#define	MACDENY_ENABLED		  276
#define	IPDENY_ENABLED		  277
#define WLAN_BLOCK_RELAY	  278

#define AUTO_MAC_CLONE		  316
#define	PPP_TTL_ENABLED		  317
#define PROTECTION_DISABLED	  318
#define MACCLONE_ENABLED	  319
#define BAND			  320
#define FIX_RATE		  321
#define WPA2_PRE_AUTH		  322
#define WPA2_CIPHER_SUITE	  323

#define	SDHCP_ENABLED		  326
#define	SDHCP_TBL_NUM		  327
#define	SDHCP_TBL		  328 /* 4 entries??*/

#define	MAX_DOWNLOAD_BANDWIDTH	  332
#define	MAX_UPLOAD_BANDWIDTH	  333
#define	REPEATER_ENABLED	  334
#define	REPEATER_SSID		  335
#define	ROAMING_THREAD		  336

/* Needed for dump util */
#define LAST_VALUE	ROAMING_THREAD

struct vser_table_entry {
		unsigned char	ip_addr[4];
		unsigned char	private_port[2];
		unsigned char	public_port[2];
		unsigned char	protocols;
		unsigned char	comment[17];
			};

struct sdhcp_tbl_entry     {
		unsigned char	mac_addr[6];
		unsigned char 	ip_addr[4];
			   };
			       
struct wan1_qos_tbl_entry	{
		unsigned char	comment[17];
		unsigned char 	upload_bandwidth[4];
		unsigned char	upload_type; /* 0 guarantee, 1 limit */
		unsigned char	download_bandwidth[4];
		unsigned char	download_type; /* 0 guarantee, 1 limit to */
		unsigned char	z;
		unsigned char	ip_addr_s1[4];
		unsigned char	ip_addr_e1[4];
		unsigned char	mac_addr_1[6];
		unsigned char	mac_addr_2[6];
		unsigned char	mac_addr_3[6];
		unsigned char	mac_addr_4[6];
		unsigned char	mac_addr_5[6];
		unsigned char	mac_addr_6[6];
		unsigned char	mac_addr_7[6];
		unsigned char	mac_addr_8[6];
		unsigned char	ip_addr_s2[4];
		unsigned char	ip_addr_e2[4];
		unsigned char	traffic_type[2];
		unsigned char	protocols; /* TCP=0 UDP=1 */
		unsigned char	public_port_s1[2];
		unsigned char	public_port_e1[2];
		unsigned char	public_port_s2[2];
		unsigned char	public_port_e2[2];
		unsigned char	public_port_s3[2];
		unsigned char	public_port_e3[2];
		unsigned char	public_port_s4[2];
		unsigned char	public_port_e4[2];
		unsigned char	public_port_s5[2];
		unsigned char	public_port_e5[2];
		unsigned char	private_port_s1[2];
		unsigned char	private_port_e1[2];
		unsigned char	private_port_s2[2];
		unsigned char	private_port_e2[2];
		unsigned char	private_port_s3[2];
		unsigned char	private_port_e3[2];
		unsigned char	private_port_s4[2];
		unsigned char	private_port_e4[2];
		unsigned char	private_port_s5[2];
		unsigned char	private_port_e5[2];
		unsigned char	unknown[2];
				};
				
struct routing_table_entry {
		unsigned char ip_addr[4];
		unsigned char netmask[4]; 
		unsigned char gateway_ip[4]; 
		unsigned char hops; 
		unsigned char iface;
			    };
			    
struct triggerport_tbl_entry {
		unsigned char port_start[2];
		unsigned char port_end[2];
		unsigned char protocols;
		unsigned char public_port_s1[2];
		unsigned char public_port_e1[2];
		unsigned char public_port_s2[2];
		unsigned char public_port_e2[2];
		unsigned char public_port_s3[2];
		unsigned char public_port_e3[2];
		unsigned char public_port_s4[2];
		unsigned char public_port_e4[2];
		unsigned char public_port_s5[2];
		unsigned char public_port_e5[2];
		unsigned char filler_gap[2];
		unsigned char public_protocols;
		unsigned char comment[17]; /* not sure, to check */
			     };
			     
struct macfilter_tbl_entry { 
		unsigned char	mac_addr[6];
		unsigned char	comment[17];
			    };
			    
struct urlb_table_entry 	 {
		unsigned char	keyword[34];
				 };
				 
struct dmz_table_entry	 {
		unsigned char	flag1;
		unsigned char	flag2;
		unsigned char	ip_public_addr[4];
		unsigned char	ip_private_addr[4];
			 };
			 
struct portfw_tbl_entry	 {
		unsigned char	ip_addr[4];
		unsigned char	port_start[2];
		unsigned char	port_end[2];
		unsigned char	protocols;
		unsigned char	comment[17];
			 };
			 
struct acpc_table_entry 	 { /* seems ok, differs from apmib.h */
		unsigned char index[2];
		unsigned char ports_bitmap[2];
		unsigned char sipaddr[4];
		unsigned char eipaddr[4];
		unsigned char protocols;
		unsigned char comment[17];
		unsigned short port[5][2];
				 };

struct portfilter_tbl_entry	{ /* portfilter seems to be unused/wrong */
		unsigned char	port_start[2];
		unsigned char	port_end[2];
		unsigned char	protocols;
		unsigned char	comment[17];
				};

struct ipfilter_tbl_entry	{
		unsigned char	ip_addr[4];
		unsigned char	protocols;
				};
				 
/* Configuration structure to be written on flash */
typedef struct {
    /* Header */
/* changed to 4 for BR-6204Wg, W.P. */    
    unsigned char	signature[4] __PACK__;
/*  removed for BR-6204Wg, W.P. */
/*    unsigned char	version __PACK__; */
    unsigned short	length __PACK__;
    
    /* LAN Configuration */
/* added for BR-6204Wg W.P.*/
    unsigned char	ssid[33] 		__PACK__;
    unsigned char	channel 		__PACK__;
    unsigned char	dont_know_1[12] 	__PACK__; /* filler */
    unsigned char	wep 			__PACK__;
    unsigned char	wep64_key1[5] 		__PACK__;
    unsigned char	wep64_key2[5] 		__PACK__;
    unsigned char	wep64_key3[5] 		__PACK__;
    unsigned char	wep64_key4[5] 		__PACK__;
    unsigned char	wep128_key1[13] 	__PACK__;
    unsigned char	wep128_key2[13] 	__PACK__;
    unsigned char	wep128_key3[13] 	__PACK__;
    unsigned char	wep128_key4[13] 	__PACK__;
    unsigned char	wep_default_key 	__PACK__;
    unsigned char	wep_key_type 		__PACK__;
    unsigned char	frag_threshold[2] 	__PACK__;
    unsigned char	rts_threshold[2] 	__PACK__;
/* dotd jest OK */
    unsigned char	supported_rates[2] 	__PACK__; /* niepewne */
    unsigned char	basic_rates[2] 		__PACK__; /* niepewne */
    unsigned char	beacon_interval[2] 	__PACK__; /* niepewne */
    unsigned char	preamble_type 		__PACK__;
    unsigned char	auth_type 		__PACK__;
    unsigned char	hidden_ssid 		__PACK__;
    unsigned char	wlan_disabled 		__PACK__;
    unsigned char	dont_know2[416] 	__PACK__;
    unsigned char	inactivity_time[2] 	__PACK__;
    unsigned char	rate_adaptive_enabled 	__PACK__;
    unsigned char	dont_know3[4] 		__PACK__;
    unsigned char	dtim_period 		__PACK__;
    unsigned char	network_type 		__PACK__;
    unsigned char	iapp_disabled 		__PACK__;
    unsigned char	ip_addr[4] 		__PACK__;
    unsigned char	subnet_mask[4] 		__PACK__;
    unsigned char 	default_gateway[4] 	__PACK__;
    unsigned char	dhcp 			__PACK__;
    unsigned char	stp_enabled 		__PACK__;
    unsigned char	super_name[31] 		__PACK__;
    unsigned char	super_password[31] 	__PACK__;
    unsigned char	user_name[31] 		__PACK__;
    unsigned char	user_password[31] 	__PACK__;
    unsigned char	dont_know4[38] 		__PACK__;
    unsigned char	dhcp_client_start[4] 	__PACK__;
    unsigned char	dhcp_client_end[4] 	__PACK__;
    unsigned char	licence 		__PACK__;
    unsigned char	wan_dhcp 		__PACK__;
    unsigned char	dns_mode 		__PACK__;
    unsigned char	dns1[4] 		__PACK__;
    unsigned char	dns2[4] 		__PACK__;
    unsigned char	dns3[4] 		__PACK__;
    unsigned char	wan_mode 		__PACK__;
    unsigned char	dynip_hostname[31] 	__PACK__;
    unsigned char	wan_mac_addr[6] 	__PACK__;
    unsigned char	wan_ip_addr[4] 		__PACK__;
    unsigned char	wan_subnet_mask[4] 	__PACK__;
    unsigned char	wan_default_gateway[4] 	__PACK__;
    unsigned char	ppp_user_name[31] 	__PACK__;
    unsigned char	ppp_password[31] 	__PACK__;
    unsigned char	ppp_idle_time[2] 	__PACK__;
    unsigned char	ppp_connect_type 	__PACK__;
    unsigned char	ppp_servname[31] 	__PACK__;
    unsigned char	ppp_mtu[2] 		__PACK__;
    unsigned char	ppp_ttl_enabled 	__PACK__;
    unsigned char	pptp_ipmode 		__PACK__;
    unsigned char	pptp_ip_addr[4] 	__PACK__;
    unsigned char	pptp_ip_mask_addr[4] 	__PACK__;
    unsigned char	pptp_def_gateway[4] 	__PACK__;
    unsigned char	pptp_gateway[4] 	__PACK__;
    unsigned char	pptp_user_name[31] 	__PACK__;
    unsigned char	pptp_password[31] 	__PACK__;
    unsigned char	pptp_idle_time[2] 	__PACK__;
    unsigned char	pptp_connect_type 	__PACK__;
    unsigned char	pptp_connt_id[31] 	__PACK__;
    unsigned char	pptp_bezeq_enabled 	__PACK__;
    unsigned char	pptp_mtu[2] 		__PACK__;
    unsigned char	l2tp_ipmode 		__PACK__;
    unsigned char	l2tp_ip_addr[4] 	__PACK__;
    unsigned char	l2tp_mask_addr[4] 	__PACK__;
    unsigned char	l2tp_defgateway[4] 	__PACK__;
    unsigned char	l2tp_gateway[31] 	__PACK__;
    unsigned char	l2tp_user_name[31] 	__PACK__;
    unsigned char	l2tp_password[31] 	__PACK__;
    unsigned char	l2tp_idle_time[2] 	__PACK__;
    unsigned char	l2tp_connect_type 	__PACK__;
    unsigned char	l2tp_mtu[2] 		__PACK__;
    unsigned char	telbp_ip_addr[4] 	__PACK__;
    unsigned char	telbp_user_name[31] 	__PACK__;
    unsigned char	telbp_password[31] 	__PACK__;
    unsigned char	telbp_enabled 		__PACK__;
/* Port forwarding table */
    unsigned char	portfw_enabled 		__PACK__;
    unsigned char	portfw_tbl_num 		__PACK__;
    struct portfw_tbl_entry portfw_tbl[16] 	__PACK__;
/* IP filter table */
    unsigned char	ipfilter_enabled 	__PACK__;
    unsigned char	ipfilter_tbl_num 	__PACK__;
    struct ipfilter_tbl_entry ipfilter_tbl[16] 	__PACK__; /* 16 is guessed */
/* DMZ table */
    unsigned char	dmz_enabled 		__PACK__;
    unsigned char	dmz_tbl_num 		__PACK__;
    struct dmz_table_entry dmz_tbl[16]		__PACK__;
/* URL blocking table */
    unsigned char	urlb_enabled 		__PACK__;
    unsigned char	urlb_tbl_num 		__PACK__;
    struct urlb_table_entry urlb_tbl[16]	__PACK__;
/* PC access table */				 
    unsigned char	acpc_enabled 		__PACK__;
    unsigned char	acpc_tbl_num 		__PACK__;
    struct acpc_table_entry acpc_tbl[20] 	__PACK__;
				 
    unsigned char	ipdeny_enabled 		__PACK__;

    unsigned char	macdeny_enabled 	__PACK__;
/* MAC filter table */
    unsigned char	macfilter_enabled 	__PACK__;
    unsigned char	macfilter_tbl_num 	__PACK__;
    struct macfilter_tbl_entry macfilter_tbl[16] __PACK__;
/* Port filter table */
    unsigned char	portfilter_enabled 	__PACK__;
    unsigned char	portfilter_tbl_num 	__PACK__;
    struct portfilter_tbl_entry portfilter_tbl[16] __PACK__;
/* Port triggers table */
    unsigned char	triggerport_enabled 	__PACK__;
    unsigned char	triggerport_tbl_num 	__PACK__;
    struct triggerport_tbl_entry triggerport_tbl[16] __PACK__;
/* Static routing table */
    unsigned char	srout_enabled 		__PACK__;
    unsigned char	srout_tbl_num 		__PACK__;
    struct routing_table_entry srout_tbl[16]	__PACK__;
/* QoS table */
    unsigned char	wan1_qos_enabled 	__PACK__;
    unsigned char	wan1_qos_tbl_num 	__PACK__;
    struct wan1_qos_tbl_entry wan1_qos_tbl[32] 	__PACK__;
/* Static DHCP leases */
    unsigned char	sdhcp_enabled 		__PACK__;
    unsigned char	sdhcp_tbl_num 		__PACK__;
    struct sdhcp_tbl_entry  sdhcp_tbl[16]	__PACK__;

    unsigned char	wlan_tran_rate[4] 	__PACK__;
    unsigned char	wlan_rate_mode 		__PACK__;
    unsigned char	wlan_cts 		__PACK__;
    unsigned char	wlan_burst 		__PACK__;
    unsigned char	wep152_key1[16] 	__PACK__;
    unsigned char	wep152_key2[16] 	__PACK__;
    unsigned char	wep152_key3[16] 	__PACK__;
    unsigned char	wep152_key4[16] 	__PACK__;
    unsigned char	dot1x_mode 		__PACK__;
    unsigned char	elan_mac_addr[6] 	__PACK__;
    unsigned char	wlan_mac_addr[6] 	__PACK__;
    
    unsigned char	protection_disabled 	__PACK__;
    unsigned char	macclone_enabled 	__PACK__;
    unsigned char	band 			__PACK__;
    unsigned char	fix_rate 		__PACK__;
    unsigned char	wpa2_pre_auth 		__PACK__;
    unsigned char	wpa2_cipher_suite 	__PACK__;
    unsigned char	wlan_set_tx 		__PACK__;
    unsigned char	ap_mode 		__PACK__;
    unsigned char	security_mode 		__PACK__;
    unsigned char	client_ip_disabled 	__PACK__;
    unsigned char	wlan_block_relay 	__PACK__;
    unsigned char	auto_mac_clone 		__PACK__;
    unsigned char	op_mode 		__PACK__;
    unsigned char	wisp_wan_id 		__PACK__;
    unsigned char	wl_linkmac1[6] 		__PACK__;
    unsigned char	wl_linkmac2[6] 		__PACK__;
    unsigned char	wl_linkmac3[6] 		__PACK__;
    unsigned char	wl_linkmac4[6] 		__PACK__;
    unsigned char	wl_linkmac5[6] 		__PACK__;
    unsigned char	wl_linkmac6[6] 		__PACK__;
    unsigned char	wds_enabled 		__PACK__;
    unsigned char	wds_encrypt 		__PACK__;
    unsigned char	wds_wep_format 		__PACK__;
    unsigned char	wds_wep_key[13] 	__PACK__; /* 13 jest niepewne, sprawdzi */
    unsigned char	wds_psk_format 		__PACK__;
    unsigned char	wds_psk[13] 		__PACK__;
    unsigned char	wlan_encrypt 		__PACK__;
    unsigned char	wlan_enable_supp_nonwpa __PACK__;
    unsigned char	wlan_supp_nonwpa 	__PACK__;
    unsigned char	wlan_wpa_auth 		__PACK__;
    unsigned char	wlan_wpa_cipher_suite 	__PACK__;
    unsigned char	wlan_wpa_psk[13] 	__PACK__;
    unsigned char	wlan_wpa_group_rekey_time[4] __PACK__;
    unsigned char	mac_auth_enabled 	__PACK__;
    unsigned char	rs_ip[4] 		__PACK__;
    unsigned char	rs_port[2] 		__PACK__;
    unsigned char	rs_password[31] 	__PACK__;
    unsigned char	rs_maxretry		__PACK__;
    unsigned char	rs_interval_time 	__PACK__;
    unsigned char	account_rs_enabled 	__PACK__;
    unsigned char	account_rs_ip[4] 	__PACK__;
    unsigned char	account_rs_port[2] 	__PACK__;
    unsigned char	account_rs_password[31] __PACK__;
    unsigned char	account_rs_update_enabled __PACK__;
    unsigned char	account_rs_update_delay __PACK__;
    unsigned char	account_rs_maxretry 	__PACK__;
    unsigned char	account_rs_interval_time __PACK__;
    unsigned char	wlan_enable_1x 		__PACK__;
    unsigned char	wlan_psk_format 	__PACK__;
    unsigned char	dhcpgatewayip_addr[4] 	__PACK__;
    unsigned char	dhcpnameserver_addr[4] 	__PACK__;
    unsigned char	domain_name[31] 	__PACK__; /* j.w. */
    unsigned char	lan_lease_time[4] 	__PACK__;
    unsigned char	wlan_macac_num 		__PACK__;
    unsigned char	wlan_macac_enabled 	__PACK__;
    unsigned char	time_zone_sel 		__PACK__;
    unsigned char	start_month[2] 		__PACK__;
    unsigned char	start_day[2] 		__PACK__;
    unsigned char	end_month[2] 		__PACK__;
    unsigned char	end_day[2] 		__PACK__;
    unsigned char	timeserver_addr[4] 	__PACK__;
    unsigned char	daylight_enable 	__PACK__;
    unsigned char	remanhost_addr[4] 	__PACK__;
    unsigned char	reman_port[2] 		__PACK__;
    unsigned char	remang_enable 		__PACK__;
    unsigned char	upnp_enable 		__PACK__;
    unsigned char	nat_enable 		__PACK__;
    unsigned char	firewall_enable 	__PACK__;
/* Virtual server table */
    unsigned char	vser_enabled 		__PACK__;
    unsigned char	vser_tbl_num 		__PACK__;
    struct vser_table_entry  vser_tbl[16]	__PACK__;
			    
    unsigned char	pod_enabled 		__PACK__;
    unsigned char	ping_enabled 		__PACK__;
    unsigned char	scan_enabled 		__PACK__;
    unsigned char	syn_enabled 		__PACK__;
    unsigned char	pod_pack[2] 		__PACK__;
    unsigned char	pod_time 		__PACK__;
    unsigned char	pod_bur[2] 		__PACK__;
    unsigned char	syn_pack[2] 		__PACK__;
    unsigned char	syn_time		__PACK__;
    unsigned char	syn_bur[2] 		__PACK__;
    unsigned char	scan_num[4] 		__PACK__;
    unsigned char	ddns_name[31] 		__PACK__;
    unsigned char	ddns_account[31] 	__PACK__;
    unsigned char	ddns_password[31]	__PACK__;
    unsigned char	ddns_enabled 		__PACK__;
    unsigned char	ddns_pvid_sel[31] 	__PACK__;
    unsigned char	app_layer_gateway[4] 	__PACK__;
    unsigned char	dhis_hostid 		__PACK__;
    unsigned char	dhis_isaddr[31] 	__PACK__;
    unsigned char	dhis_password[31] 	__PACK__;
    unsigned char	dhis_auth_p1[31] 	__PACK__;
    unsigned char	dhis_auth_p2[31] 	__PACK__;
    unsigned char	dhis_auth_q1[31] 	__PACK__;
    unsigned char	dhis_auth_q2[31] 	__PACK__;
    unsigned char	dhis_select 		__PACK__;
    unsigned char	max_download_bandwidth[4] __PACK__;
    unsigned char	max_upload_bandwidth[4] __PACK__;
    unsigned char	repeater_enabled 	__PACK__;
    unsigned char	repeater_ssid[33] 	__PACK__;
    unsigned char	roaming_thread 		__PACK__;
    unsigned char	ps_enable 		__PACK__;
    unsigned char	ps_ippenable 		__PACK__;
    unsigned char	ps_lprenable 		__PACK__;
    unsigned char	ps_name[31] 		__PACK__;
    unsigned char	ps_port1name[31] 	__PACK__;
    unsigned char	ps_port2name[31] 	__PACK__;

    unsigned char	filler4[21895] 		__PACK__;
    unsigned char	checksum		__PACK__;
    
/* end modifications. W.P. */    
    
} CONFIG_STRUCT;

/* Hardware configuration (taken from default firmware) */
typedef struct {
    /*-----> Header <----*/
    unsigned char	signature[4] __PACK__; 
    unsigned short	len __PACK__;
    /*----> EOH <----- */
    
    unsigned char	boardVer __PACK__;
    unsigned char	nic0Addr[6] __PACK__;
    unsigned char	nic1Addr[6] __PACK__;
    /* added for BR-6204Wg W.P.*/
    unsigned char	wlanAddr [6] __PACK__;
    unsigned char	txPowerCCK [14] __PACK__;
    unsigned char	txPowerOFDM [162] __PACK__;
    unsigned char	RegDomain __PACK__;
    unsigned char	RFtype __PACK__;
    unsigned char	AntDiversity __PACK__;
    unsigned char	TxAnt __PACK__;
    unsigned char	CCAmode __PACK__;
    unsigned char	WlanLedType __PACK__;
    unsigned char	InitGain __PACK__;
} HW_CONFIG_STRUCT;


/* Type IDs for requesting a read or write */
typedef enum {
    BYTE_T,
    WORD_T,
    STRING_T,
    IP_ADDR_T,
    BYTE_ARRAY_T, 
    LONG_T, 
    DHCP_ARRAY_T,
    PORTFW_ARRAY_T,
    MACFILTER_ARRAY_T,
    VSER_ARRAY_T, 
    SROUT_ARRAY_T, 
    DMZ_ARRAY_T,
    URLB_ARRAY_T, 
    TRIGGERPORT_ARRAY_T,
    PORTFILTER_ARRAY_T,
    WAN1_QOS_ARRAY_T,
    ACPC_ARRAY_T, 
    IPFILTER_ARRAY_T, 
    ALG_BITMAP_T,
    SCAN_BITMAP_T
} TYPE_T;

/* Descriptor of each entry for high level use */
typedef struct {
    int			id;
    char		name[32];
    TYPE_T		type;
    int			offset;
    int			size;
    int			prt_size; /* max.printable size */
    int			prt_row_size; /* max size of table row when printed */
} APP_DESC;

/* Macros for making the config table easier */
#define FIELD_OFFSET(type,field)		((unsigned long)(long*)&(((type*)0)->field))
#define OFFSET(field)				((int)FIELD_OFFSET(CONFIG_STRUCT,field))
#define SIZE(field)				sizeof(((CONFIG_STRUCT*)0)->field)

/* printable size definitions for types */
#define PRT_SIZE_OF_BYTE_T 4
#define PRT_SIZE_OF_IP_ADDR_T 16
#define PRT_SIZE_OF_MAC_ADDR_T 19
#define PRT_SIZE_OF_WORD_T 6
#define PRT_SIZE_OF_LONG_T 13

int configInit(void);
int configUpdate(void);
int configGet(int id, void *ptr);
int configSet(int id, void *ptr, int size);
int configCreateDefault(void);
int configGetIDByString(char *string);
int configGetTypeByID(int id);
int configGetSizeByID(int id);
int configGet_PRT_SizeByID(int id);
int configGet_PRT_RowsizeSizeByID(int id);
char *configGetNameByID(int id);
char *configGetMACAddr0(void);
char *configGetMACAddr1(void);
char *configGetWlanAddr(void);
char configGetBoardVersion(void);
int configSetValueByString(int id, char *value);
#endif
