/*
 * Main configuration manipulation program for BR-6104K - Part of AMiLDA Linux
 * Copyright (C)2005 by Sergio Aguayo
 *
 * Covered by the Mozilla Public License v1.1
 * See the license file for details
 */
#include <stdio.h>
#include "config.h"

#define DEBUG(x) printf(x); fflush(stdout);

int getmac (int argc, char *argv[]) {
    unsigned char *macaddr;
    int inface;
    
    if (argc < 1) {
	printf("Wrong number of parameters.\n");
	return 1;
    }
    
    sscanf(argv[0], "%i", &inface);
    
    if (inface < 0 || inface > 2) {
/* changed to 2 for BR-6204Wg W.P */
	printf("Bad interface number (0-2).\n");
	return 1;
    }
    
    if (inface == 0) macaddr = configGetMACAddr0();
    else 
/* changed to support 0-2 for BR-6204Wg W.P */    
	if (inface == 2) macaddr = configGetWlanAddr();
	else macaddr = configGetMACAddr1();
        
    printf("NIC%i_ADDR='%.2X%.2X%.2X%.2X%.2X%.2X'\n", inface,   (unsigned int)macaddr[0],
								(unsigned int)macaddr[1], 
								(unsigned int)macaddr[2],
								(unsigned int)macaddr[3], 
								(unsigned int)macaddr[4], 
								(unsigned int)macaddr[5]);
    
    return 0;
}

int check (int argc, char *argv[]) {
    if (argc != 0) return 1;
    return !configInit();
}

int outputValueByID(int id) {
    int type;
    void *data;
    int datalen;
    int PRT_len;
    int i;
    
    if (id <= 0) return 0;
    if (id > LAST_VALUE) return 0; /* atempt to read beyond max value defined */
    
    type = configGetTypeByID(id);    
    datalen = configGetSizeByID(id);
    PRT_len = configGet_PRT_SizeByID(id);

    if (datalen == 0) return 0;

    data = malloc(PRT_len);
    if (data == NULL) {
	free(data);
	return 0;
		      }
    bzero(data, PRT_len);

    if (!configGet(id, data)) {
	free(data);
	return 0;
    }
/* WP - approach used: simple type: print name, complex type: print whole table */

    switch (type) {
	case DHCP_ARRAY_T:
	    printf("%s", data);
	    break;

	case PORTFW_ARRAY_T:
	    printf("%s", data);
	    break;

	case VSER_ARRAY_T:
	    printf("%s", data);
	    break;

	case SROUT_ARRAY_T:
	    printf("%s", data);
	    break;

	case DMZ_ARRAY_T:
	    printf("%s", data);
	    break;

	case URLB_ARRAY_T:
	    printf("%s", data);
	    break;

	case MACFILTER_ARRAY_T:
	    printf("%s", data);
	    break;

	case TRIGGERPORT_ARRAY_T:
	    printf("%s", data);
	    break;

	case PORTFILTER_ARRAY_T:
	    printf("%s", data);
	    break;

	case ACPC_ARRAY_T:
	    printf("%s", data);
	    break;
	    
	case WAN1_QOS_ARRAY_T:
	    printf("%s", data);
	    break;

	case IPFILTER_ARRAY_T:
	    printf("%s", data);
	    break;
	    
	default:
	    printf("%s=", configGetNameByID(id));
	    break;
		  }
/* end WP */    
    switch (type) {
	case BYTE_T:
	    printf("%u\n", (unsigned int)(*(unsigned char*)data));
	    break;
	case WORD_T:
	    printf("%u\n", (unsigned int)(*(unsigned short*)data));
	    break;
	case STRING_T:
	    printf("'%s'\n", (unsigned char*)data);
	    break;
	case IP_ADDR_T:
	    printf("'%s'\n", data);
	    break;
	case BYTE_ARRAY_T:
	    for (i = 0; i < datalen; i++) {
		printf("%.2X", (unsigned int)((unsigned char*)data)[i]);
	    }
	    printf("\n");
	    break;
/* WP */
	case LONG_T:
	    printf("%s\n", data);
	    break;

	case ALG_BITMAP_T:
	    printf("%s\n", data);
	    break;

	case SCAN_BITMAP_T:
	    printf("%s\n", data);
	    break;
/* WP */	    
    }
    
    free(data);

    return 1;    
}


int reset(int argc, char* argv[]) {
    if (argc > 0) return 1;
    
    return !configCreateDefault();
}

int get(int argc, char* argv[]) {
    int id;
    
    if (argc <= 0 || argc > 1) {
	printf("Wrong number of arguments\n");
	return 1;
    }
    
    id = configGetIDByString(argv[0]);
    if (id == 0) {
	printf("Invalid token.\n");
	return 1;
    }

    if (!outputValueByID(id)) {
	printf("Error obtaining value.\n");
	return 1;
    }
    
    return 0;
}

int set(int argc, char *argv[]) {
    int id;
    char *ptr;
    FILE *fh;
    
    if (argc <= 0 || argc > 2) {
	printf("Invalid number of arguments.\n");
	return 1;
    }
    
    id = configGetIDByString(argv[0]);
    if (id == 0) {
	printf("Illegal token.\n");
	return 1;
    }
    if (!configSetValueByString(id, argv[1])) {
	printf("Error setting value.\n");
	return 1;
    }
/* debug */
/*    fh = fopen("/tmp/xxx.yyy", "wb");
    fwrite("Attempt to UPDATE", sizeof("Attempt to UPDATE"), 1, fh);
    close(fh);*/
    configUpdate();
/* end debug */
    
    return 0;
}

int dump(int argc, char *argv[]) {
    int i;
    
    if (argc > 0) return 1;

    for (i = 1; i <= LAST_VALUE; i++) {
	fflush(stdout);
	outputValueByID(i);
    }
    
    return 0;
}

int test(int argc, char *argv[]) {
    int id;
    char *wiersz;
    char *tmp;
    unsigned char *tmp2;
    unsigned char *dane;
    
    id = configGetIDByString("IP_ADDR");
    wiersz = malloc(configGet_PRT_SizeByID(id)*2+1);
    dane = malloc((configGetSizeByID(id))*2);
    bzero(wiersz, configGet_PRT_SizeByID(id)*2+1);
    bzero(dane, (configGetSizeByID(id))*2);

    tmp2 = dane;
    tmp = wiersz;
/*    configGet(id, dane);*/
    dane[0] = 192;
    dane[1] = 168;
    dane[2] = 1;
    dane[3] = 99;
    dane[4] = 172;
    dane[5] = 158;
    dane[6] = 10;
    dane[7] = 9;
    
    sPrintIP_Addr(&dane, &wiersz);
    sPrintLong(&dane, &wiersz);
/*    sPrintIP_Addr(&dane, &wiersz);*/
    
    printf("%s\n", tmp);
    
    free(tmp2);
    free(tmp);
        
    return 0;
}

int main (int argc, char *argv[]) {

    unsigned char temp1;

    if (argc < 2) {
	printf("Usage:\n\n");
	printf(" %s command [parameters]\n\n", argv[0]);
	printf("Where command is one of the following:\n");
	printf("  check:  Checks the signatures of the\n");
	printf("          configuration structure. Returns\n");
	printf("          0 if successful, else non-zero.\n");
	printf("  reset:  Writes the default configuration\n");
	printf("          to the flash memory.\n");
	printf("  get:    Obtains the specified value from\n");
	printf("          the flash memory.\n");
	printf("  set:    Sets the specified value into the\n");
	printf("          flash memory. The option is the first\n");
	printf("          parameter, the value to set is the 2nd.\n");
	printf("  dump:   Dumps all the configuration values\n");
	printf("  or all: into a format that can be included\n");
	printf("          from a bash script.\n");
	printf("  getmac: Gets the MAC address of the specified\n");
	printf("          network interface (0 or 1,2 for WLAN).\n\n");
	return 1;
    }
    
    
    if (!strcmp(argv[1], "check")) return check(argc-2, &argv[2]);
    if (!strcmp(argv[1], "reset")) return reset(argc-2, &argv[2]);

    temp1 = configInit();
    
    if (!temp1) {
	printf("WARNING: Invalid configuration found.\n");
	printf("Use 'reset' parameter before proceeding.\n");
	return 1;
    }
    
    if (!strcmp(argv[1], "get")) return get(argc-2, &argv[2]);
    if (!strcmp(argv[1], "set")) return set(argc-2, &argv[2]);
    if (!strcmp(argv[1], "dump")) return dump(argc-2, &argv[2]);
/* WP */
    if (!strcmp(argv[1], "all")) return dump(argc-2, &argv[2]);
    if (!strcmp(argv[1], "test")) return test(argc-2, &argv[2]);
/* WP */    
    if (!strcmp(argv[1], "getmac")) return getmac(argc-2, &argv[2]);
    
    return 1;
}
