/*
 * MTD Block file manipulation function for BR-6104K - Part of AMiLDA Linux
 * Copyright (C)2005 by Sergio Aguayo
 *
 * Covered by the Mozilla Public License v1.1
 * See the license file for details
 */

#include <stdio.h>
#include "mtd.h"


int mtdRead(void *ptr, unsigned int size, unsigned int offset) {
    FILE *fh;
    
    fh = fopen(MTD_DEVICE, "rb");
    if (fh == NULL) return 0;
    
    if (fseek(fh, offset, SEEK_SET) == -1) {
	fclose(fh);
	return 0;
    }
    
    if (fread(ptr, size, 1, fh) != 1) {
	fclose(fh);
	return 0;
    }
    
    fclose(fh);
    
    return 1;
}

int mtdWrite(void *ptr, unsigned int size, unsigned int offset) {
    FILE *fh;
    
    /* Only allow writing to configuration area (avoids accidental overwrite of loader for example) */
    if (offset < 0x8000 || (offset+size) > 0x10000) return 0;
    
    fh = fopen(MTD_DEVICE, "wb");
    if (fh == NULL) return 0;
    
    if (fseek(fh, offset, SEEK_SET) == -1) {
	fclose(fh);
	return 0;
    }
    
    if (fwrite(ptr, size, 1, fh) != 1) {
	fclose(fh);
	return 0;
    }
    
    fclose(fh);
    
    return 1;
}
