/*
 * FILE NAME rtl_gpio.c
 *
 * BRIEF MODULE DESCRIPTION
 *  GPIO For Flash Reload Default 
 *
 *  Author: Realtek , Inc.  
 *          sc_yang@realtek.com.tw
 *
 * Copyright 2001 MontaVista Software Inc.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE	LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */



#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <asm/errno.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <linux/miscdevice.h>
#include <linux/pci.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/sched.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/reboot.h>
//#include <linux/kmod.h>
#ifdef CONFIG_RTL_EB8186
	#define RTL_GPIO_PABDIR 	0xbd010124
	#define RTL_GPIO_PABDATA 	0xbd010120
	#define RESET_PIN	0
	#define LED_PIN	2
#else
	#define RTL_GPIO_PABDIR 	0xbd010040
	#define RTL_GPIO_PABDATA 	0xbd010044
	#define RTL_WLAN_CR9346		0xbd400050
	#define RTL_WLAN_CONFIG0	0xbd400051
	#define RTL_WLAN_PSR		0xbd40005e
	#define RESET_PIN	10
#endif

#define PROBE_NULL	0
#define PROBE_ACTIVE	1
#define PROBE_RESET	2
#define PROBE_RELOAD	3
#define RTL_R32(addr)   (*(volatile unsigned long *)(addr))
#define RTL_W32(addr, l)   ((*(volatile unsigned long*)(addr)) = (l))
#define RTL_R8(addr)   (*(volatile unsigned char*)(addr))
#define RTL_W8(addr, l)   ((*(volatile unsigned char*)(addr)) = (l))

//#define  GPIO_DEBUG
#ifdef GPIO_DEBUG
/* note: prints function name for you */
#  define DPRINTK(fmt, args...) printk("%s: " fmt, __FUNCTION__ , ## args)
#else
#  define DPRINTK(fmt, args...)
#endif

static struct timer_list probe_timer;
static struct timer_list led_timer;		//Erwin
static unsigned int    probe_counter;
static unsigned int    probe_state;
static unsigned int    wlan_counter;	//Erwin
static unsigned int    power_counter;	//Erwin
//static int led_flash[20]={10,5,10,50,2,5,5,5,70,10,50,20,42,10,43,60,10,20,13,39};	//Erwin
static int led_flash[30]={20,10,100,5,5,150,100,5,5,50,20,50,50,20,60,5,20,10,30,10,5,10,50,2,5,5,5,70,10,50};//Erwin

#ifndef CONFIG_RTL_EB8186
unsigned int led0enable;
#endif

static int reset_flash_default(void *data)
{
	char *argv[3], *envp[1] = {NULL};
	int i = 0;
	int reset_default=(int) data;

	argv[i++] = "/bin/flash";
	argv[i++] = "reset";
	argv[i] = NULL;

	if(reset_default)
		if (call_usermodehelper(argv[0], argv, envp,1))
			DPRINTK("failed to Reset to default\n");
	machine_restart(0);
	return 0;
}

static void rtl_gpio_flash(unsigned long data)
{
	RTL_W32(RTL_GPIO_PABDIR, (RTL_R32(RTL_GPIO_PABDIR) | 0x8));	//Erwin

//	if (led_flash[wlan_counter%20]==1)	{
	if ((wlan_counter%2) == 1)	{
		RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) & ~0x8));	//Erwin
	}
	else {
		RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) | 0x8));	//Erwin
	}
	wlan_counter++;
	mod_timer(&led_timer, jiffies + led_flash[wlan_counter%30]);
}

static void rtl_gpio_timer(unsigned long data)
{
	unsigned int pressed=1;

	// Set GPIOA pin 10(8181),0 (8186)as input pin
	RTL_W32(RTL_GPIO_PABDIR, (RTL_R32(RTL_GPIO_PABDIR) & (~(1 << RESET_PIN))));
#ifdef CONFIG_RTL_EB8186
	// Set GPIOA  ping 2 as output pin for led
	RTL_W32(RTL_GPIO_PABDIR, (RTL_R32(RTL_GPIO_PABDIR) | ((1 << LED_PIN))));
	RTL_W32(RTL_GPIO_PABDIR, (RTL_R32(RTL_GPIO_PABDIR) | 0x4));	//Erwin
#endif


	if ((RTL_R32(RTL_GPIO_PABDATA) & (1 << RESET_PIN))){
		pressed = 0;
#ifdef CONFIG_RTL_EB8186
		//turn off LED0
		RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) | ((1 << LED_PIN))));
#endif
	}
	else
	{
		DPRINTK("Key pressed %d!\n", probe_counter+1);
	}


	if (power_counter<=16) {
		if ((power_counter%2) == 1)	{
			RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) & ~0x4));	//Erwin
		}
		else {
			RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) | 0x4));	//Erwin
		}
		power_counter++;
	}
	else
		RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) & ~0x4));	//Erwin
	


	
	if (probe_state == PROBE_NULL)
	{
		if (pressed)
		{
			probe_state = PROBE_ACTIVE;
			probe_counter++;
		}
		else
			probe_counter = 0;
	}
	else if (probe_state == PROBE_ACTIVE)
	{
		if (pressed)
		{
			probe_counter++;
			if ((probe_counter >=2 ) && (probe_counter <=5))
			{
#ifndef CONFIG_RTL_EB8186
				led0enable=1;
				// turn on LED0
				RTL_W8(RTL_WLAN_CR9346, 0xc0);
				RTL_W8(RTL_WLAN_CONFIG0, 0x10);
				RTL_W8(RTL_WLAN_PSR,(RTL_R8(RTL_WLAN_PSR)&0xEF));
#else
				DPRINTK("2-5 turn on led\n");
				//turn on LED0
				RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) & (~(1 << LED_PIN))));
#endif
			  		
			}
			else if (probe_counter >= 5)
			{
#ifndef CONFIG_RTL_EB8186
				led0enable=1;
				RTL_W8(RTL_WLAN_CR9346, 0xc0);
				RTL_W8(RTL_WLAN_CONFIG0, 0x10);
				// sparkling LED0
				if (probe_counter & 1)
					RTL_W8(RTL_WLAN_PSR, (RTL_R8(RTL_WLAN_PSR)|0x10));
				else
					RTL_W8(RTL_WLAN_PSR, (RTL_R8(RTL_WLAN_PSR)&0xEF));
#else
				// sparkling LED0
				DPRINTK(">5 \n");
				if (probe_counter & 1)	{
					RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) | ((1 << LED_PIN))));
				}
				else {
					RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) & (~(1 << LED_PIN))));
				}

#endif

			}
		}
		else
		{
			if (probe_counter < 2)
			{
				probe_state = PROBE_NULL;
				probe_counter = 0;
				DPRINTK("<2 \n");
			}
			else if (probe_counter >=5)
			{
#ifndef CONFIG_RTL_EB8186
				led0enable=1;
#endif
				//reload default
//				printk("Going to Reload Default\n");
				DPRINTK("Going to Reload Default\n");
				kernel_thread(reset_flash_default, 1, SIGCHLD);
				return;

			}
			else
			{
#ifndef CONFIG_RTL_EB8186
				RTL_W8(RTL_WLAN_PSR , (RTL_R8(RTL_WLAN_PSR)|0x10) );
				RTL_W8(RTL_WLAN_CONFIG0, (RTL_R8(RTL_WLAN_CONFIG0)&(~0x10)) );
#endif
				DPRINTK("2-5 reset\n");
				kernel_thread(reset_flash_default, 0, SIGCHLD);
//				kernel_thread(reset_flash_default, 1, SIGCHLD);
				return;
			}
		}

	}
	mod_timer(&probe_timer, jiffies + 100);
}
int __init rtl_gpio_init(void)
{
	DPRINTK("Realtek GPIO Driver for Flash Reload Default\n");
	init_timer (&probe_timer);
	probe_counter = 0;
	probe_state = NULL ;
	probe_timer.expires=jiffies + 100;
        probe_timer.data =(unsigned long) NULL ;
        probe_timer.function = &rtl_gpio_timer;
        mod_timer(&probe_timer, jiffies + 100);	


	init_timer (&led_timer);
	led_timer.expires=jiffies+10;
    led_timer.data =(unsigned long) NULL ;
    led_timer.function = &rtl_gpio_flash;
    mod_timer(&led_timer, jiffies+10);	

		
	return 0;
}

static void __exit rtl_gpio_exit(void)
{
	DPRINTK("Unload Realtek GPIO Driver \n");
	del_timer_sync(&probe_timer);
	del_timer_sync(&led_timer);
}

module_exit(rtl_gpio_exit);
module_init(rtl_gpio_init);

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("GPIO driver for Reload default");
